# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client
from tempest.lib import exceptions


class TokenClient(rest_client.RestClient):

    def __init__(self, auth_url, disable_ssl_certificate_validation=None,
                 ca_certs=None, trace_requests=None, **kwargs):
        """Initialises the Token client

        :param auth_url: URL to which the token request is sent
        :param disable_ssl_certificate_validation: pass-through to rest client
        :param ca_certs: pass-through to rest client
        :param trace_requests: pass-through to rest client
        :param kwargs: any extra parameter to pass through the rest client.
               region, service and auth_provider will be ignored, if passed,
               as they are not meaningful for token client
        """
        dscv = disable_ssl_certificate_validation
        # NOTE(andreaf) region, service and auth_provider are passed
        # positionally with None. Having them in kwargs would raise a
        # "multiple values for keyword arguments" error
        for unwanted_kwargs in ['region', 'service', 'auth_provider']:
            kwargs.pop(unwanted_kwargs, None)
        super(TokenClient, self).__init__(
            None, None, None, disable_ssl_certificate_validation=dscv,
            ca_certs=ca_certs, trace_requests=trace_requests, **kwargs)

        if auth_url is None:
            raise exceptions.IdentityError("Couldn't determine auth_url")

        # Normalize URI to ensure /tokens is in it.
        if 'tokens' not in auth_url:
            auth_url = auth_url.rstrip('/') + '/tokens'

        self.auth_url = auth_url

    def auth(self, user, password, tenant=None):
        creds = {
            'auth': {
                'passwordCredentials': {
                    'username': user,
                    'password': password,
                },
            }
        }

        if tenant:
            creds['auth']['tenantName'] = tenant

        body = json.dumps(creds, sort_keys=True)
        resp, body = self.post(self.auth_url, body=body)
        self.expected_success(200, resp.status)

        return rest_client.ResponseBody(resp, body['access'])

    def auth_token(self, token_id, tenant=None):
        creds = {
            'auth': {
                'token': {
                    'id': token_id,
                },
            }
        }

        if tenant:
            creds['auth']['tenantName'] = tenant

        body = json.dumps(creds)
        resp, body = self.post(self.auth_url, body=body)
        self.expected_success(200, resp.status)

        return rest_client.ResponseBody(resp, body['access'])

    def request(self, method, url, extra_headers=False, headers=None,
                body=None, chunked=False):
        """A simple HTTP request interface.

        Note: this overloads the `request` method from the parent class and
        thus must implement the same method signature.
        """
        if headers is None:
            headers = self.get_headers(accept_type="json")
        elif extra_headers:
            try:
                headers.update(self.get_headers(accept_type="json"))
            except (ValueError, TypeError):
                headers = self.get_headers(accept_type="json")

        resp, resp_body = self.raw_request(url, method,
                                           headers=headers, body=body,
                                           log_req_body='<omitted>')

        if resp.status in [401, 403]:
            resp_body = json.loads(resp_body)
            raise exceptions.Unauthorized(resp_body['error']['message'])
        elif resp.status not in [200, 201]:
            raise exceptions.IdentityError(
                'Unexpected status code {0}'.format(resp.status))

        return resp, json.loads(resp_body)

    def get_token(self, user, password, tenant, auth_data=False):
        """Returns (token id, token data) for supplied credentials."""
        body = self.auth(user, password, tenant)

        if auth_data:
            return body['token']['id'], body
        else:
            return body['token']['id']



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class VolumesSnapshotListTestJSON(base.BaseVolumeTest):
    """Test listing volume snapshots"""

    @classmethod
    def skip_checks(cls):
        super(VolumesSnapshotListTestJSON, cls).skip_checks()
        if not CONF.volume_feature_enabled.snapshot:
            raise cls.skipException("Cinder volume snapshots are disabled")

    @classmethod
    def resource_setup(cls):
        super(VolumesSnapshotListTestJSON, cls).resource_setup()
        volume_origin = cls.create_volume()

        # Create snapshots with params
        for _ in range(3):
            snapshot = cls.create_snapshot(volume_origin['id'])
        cls.snapshot = snapshot

    def _list_by_param_values_and_assert(self, with_detail=False, **params):
        """list or list_details with given params and validates result."""

        fetched_snap_list = self.snapshots_client.list_snapshots(
            detail=with_detail, **params)['snapshots']

        # Validating params of fetched snapshots
        for snap in fetched_snap_list:
            for key in params:
                msg = "Failed to list snapshots %s by %s" % \
                      ('details' if with_detail else '', key)
                self.assertEqual(params[key], snap[key], msg)

    def _list_snapshots_by_param_limit(self, limit, expected_elements):
        """list snapshots by limit param"""

        # Get snapshots list using limit parameter
        fetched_snap_list = self.snapshots_client.list_snapshots(
            limit=limit)['snapshots']
        # Validating filtered snapshots length equals to expected_elements
        self.assertEqual(expected_elements, len(fetched_snap_list))

    @decorators.idempotent_id('59f41f43-aebf-48a9-ab5d-d76340fab32b')
    def test_snapshots_list_with_params(self):
        """Test listing snapshots with params"""

        # Verify list snapshots by display_name filter
        params = {'name': self.snapshot['name']}
        self._list_by_param_values_and_assert(**params)

        # Verify list snapshots by status filter
        params = {'status': 'available'}
        self._list_by_param_values_and_assert(**params)

        # Verify list snapshots by status and display name filter
        params = {'status': 'available',
                  'name': self.snapshot['name']}
        self._list_by_param_values_and_assert(**params)

    @decorators.idempotent_id('220a1022-1fcd-4a74-a7bd-6b859156cda2')
    def test_snapshots_list_details_with_params(self):
        """Test listing snapshot details with params"""

        # Verify list snapshot details by display_name filter
        params = {'name': self.snapshot['name']}
        self._list_by_param_values_and_assert(with_detail=True, **params)
        # Verify list snapshot details by status filter
        params = {'status': 'available'}
        self._list_by_param_values_and_assert(with_detail=True, **params)
        # Verify list snapshot details by status and display name filter
        params = {'status': 'available',
                  'name': self.snapshot['name']}
        self._list_by_param_values_and_assert(with_detail=True, **params)

    @decorators.idempotent_id('db4d8e0a-7a2e-41cc-a712-961f6844e896')
    def test_snapshot_list_param_limit(self):
        """Test listing snapshot with limit returns the limited elements

        If listing snapshots with limit=1, then 1 snapshot is returned.
        """
        self._list_snapshots_by_param_limit(limit=1, expected_elements=1)

    @decorators.idempotent_id('a1427f61-420e-48a5-b6e3-0b394fa95400')
    def test_snapshot_list_param_limit_equals_infinite(self):
        """Test listing snapshot with infinite limit

        If listing snapshots with limit greater than the count of all
        snapshots, then all snapshots are returned.
        """
        snap_list = self.snapshots_client.list_snapshots()['snapshots']
        self._list_snapshots_by_param_limit(limit=100000,
                                            expected_elements=len(snap_list))

    @decorators.idempotent_id('e3b44b7f-ae87-45b5-8a8c-66110eb24d0a')
    def test_snapshot_list_param_limit_equals_zero(self):
        """Test listing snapshot with zero limit should return empty list"""
        self._list_snapshots_by_param_limit(limit=0, expected_elements=0)

    def _list_snapshots_param_sort(self, sort_key, sort_dir):
        snap_list = self.snapshots_client.list_snapshots(
            sort_key=sort_key, sort_dir=sort_dir)['snapshots']
        self.assertNotEmpty(snap_list)
        if sort_key == 'display_name':
            sort_key = 'name'
        # Note: On Cinder API, 'display_name' works as a sort key
        # on a request, a volume name appears as 'name' on the response.
        # So Tempest needs to change the key name here for this inconsistent
        # API behavior.
        sorted_list = [snapshot[sort_key] for snapshot in snap_list]
        msg = 'The list of snapshots was not sorted correctly.'
        self.assertEqual(sorted(sorted_list, reverse=(sort_dir == 'desc')),
                         sorted_list, msg)

    @decorators.idempotent_id('c5513ada-64c1-4d28-83b9-af3307ec1388')
    def test_snapshot_list_param_sort_id_asc(self):
        """Test listing snapshots sort by id ascendingly"""
        self._list_snapshots_param_sort(sort_key='id', sort_dir='asc')

    @decorators.idempotent_id('8a7fe058-0b41-402a-8afd-2dbc5a4a718b')
    def test_snapshot_list_param_sort_id_desc(self):
        """Test listing snapshots sort by id descendingly"""
        self._list_snapshots_param_sort(sort_key='id', sort_dir='desc')

    @decorators.idempotent_id('4052c3a0-2415-440a-a8cc-305a875331b0')
    def test_snapshot_list_param_sort_created_at_asc(self):
        """Test listing snapshots sort by created_at ascendingly"""
        self._list_snapshots_param_sort(sort_key='created_at', sort_dir='asc')

    @decorators.idempotent_id('dcbbe24a-f3c0-4ec8-9274-55d48db8d1cf')
    def test_snapshot_list_param_sort_created_at_desc(self):
        """Test listing snapshots sort by created_at descendingly"""
        self._list_snapshots_param_sort(sort_key='created_at', sort_dir='desc')

    @decorators.idempotent_id('d58b5fed-0c37-42d3-8c5d-39014ac13c00')
    def test_snapshot_list_param_sort_name_asc(self):
        """Test listing snapshots sort by display_name ascendingly"""
        self._list_snapshots_param_sort(sort_key='display_name',
                                        sort_dir='asc')

    @decorators.idempotent_id('96ba6f4d-1f18-47e1-b4bc-76edc6c21250')
    def test_snapshot_list_param_sort_name_desc(self):
        """Test listing snapshots sort by display_name descendingly"""
        self._list_snapshots_param_sort(sort_key='display_name',
                                        sort_dir='desc')

    @decorators.idempotent_id('05489dde-44bc-4961-a1f5-3ce7ee7824f7')
    def test_snapshot_list_param_marker(self):
        """Test listing snapshots with marker

        The list of snapshots should end before the provided marker
        """
        snap_list = self.snapshots_client.list_snapshots()['snapshots']
        # list_snapshots will take the reverse order as they are created.
        snapshot_id_list = [snap['id'] for snap in snap_list][::-1]

        params = {'marker': snapshot_id_list[1]}
        snap_list = self.snapshots_client.list_snapshots(**params)['snapshots']
        fetched_list_id = [snap['id'] for snap in snap_list]
        # Verify the list of snapshots ends before the provided
        # marker(second snapshot), therefore only the first snapshot
        # should displayed.
        self.assertEqual(snapshot_id_list[:1], fetched_list_id)

    @decorators.idempotent_id('ca96d551-17c6-4e11-b0e8-52d3bb8a63c7')
    def test_snapshot_list_param_offset(self):
        """Test listing snapshots with offset and limit

        If listing snapshots with offset=2 and limit=3, then at most 3(limit)
        snapshots located in the position 2(offset) in the all snapshots list
        should be returned.
        (The items in the all snapshots list start from position 0.)
        """
        params = {'offset': 2, 'limit': 3}
        snap_list = self.snapshots_client.list_snapshots(**params)['snapshots']
        # Verify the list of snapshots skip offset=2 from the first element
        # (total 3 elements), therefore only one snapshot should display
        self.assertEqual(1, len(snap_list))



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

list_extensions = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'extensions': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'updated': parameter_types.date_time,
                        'name': {'type': 'string'},
                        'links': {'type': 'array'},
                        'namespace': {
                            'type': 'string',
                            'format': 'uri'
                        },
                        'alias': {'type': 'string'},
                        'description': {'type': 'string'}
                    },
                    'additionalProperties': False,
                    'required': ['updated', 'name', 'links', 'namespace',
                                 'alias', 'description']
                }
            }
        },
        'additionalProperties': False,
        'required': ['extensions']
    }
}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import re
import time

from oslo_log import log as logging

from tempest import config
from tempest import exceptions
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions as lib_exc

CONF = config.CONF
LOG = logging.getLogger(__name__)


def _get_task_state(body):
    return body.get('OS-EXT-STS:task_state', None)


# NOTE(afazekas): This function needs to know a token and a subject.
def wait_for_server_status(client, server_id, status, ready_wait=True,
                           extra_timeout=0, raise_on_error=True,
                           request_id=None):
    """Waits for a server to reach a given status."""

    # NOTE(afazekas): UNKNOWN status possible on ERROR
    # or in a very early stage.
    body = client.show_server(server_id)['server']
    old_status = server_status = body['status']
    old_task_state = task_state = _get_task_state(body)
    start_time = int(time.time())
    timeout = client.build_timeout + extra_timeout
    while True:
        # NOTE(afazekas): Now the BUILD status only reached
        # between the UNKNOWN->ACTIVE transition.
        # TODO(afazekas): enumerate and validate the stable status set
        if status == 'BUILD' and server_status != 'UNKNOWN':
            return body
        if server_status == status:
            if ready_wait:
                if status == 'BUILD':
                    return body
                # NOTE(afazekas): The instance is in "ready for action state"
                # when no task in progress
                if task_state is None:
                    # without state api extension 3 sec usually enough
                    time.sleep(CONF.compute.ready_wait)
                    return body
            else:
                return body

        time.sleep(client.build_interval)
        body = client.show_server(server_id)['server']
        server_status = body['status']
        task_state = _get_task_state(body)
        if (server_status != old_status) or (task_state != old_task_state):
            LOG.info('State transition "%s" ==> "%s" after %d second wait',
                     '/'.join((old_status, str(old_task_state))),
                     '/'.join((server_status, str(task_state))),
                     time.time() - start_time)
        if (server_status == 'ERROR') and raise_on_error:
            details = ''
            if 'fault' in body:
                details += 'Fault: %s.' % body['fault']
            if request_id:
                details += ' Request ID of server operation performed before'
                details += ' checking the server status %s.' % request_id
            raise exceptions.BuildErrorException(details, server_id=server_id)

        timed_out = int(time.time()) - start_time >= timeout

        if timed_out:
            expected_task_state = 'None' if ready_wait else 'n/a'
            message = ('Server %(server_id)s failed to reach %(status)s '
                       'status and task state "%(expected_task_state)s" '
                       'within the required time (%(timeout)s s).' %
                       {'server_id': server_id,
                        'status': status,
                        'expected_task_state': expected_task_state,
                        'timeout': timeout})
            if request_id:
                message += ' Request ID of server operation performed before'
                message += ' checking the server status %s.' % request_id
            message += ' Current status: %s.' % server_status
            message += ' Current task state: %s.' % task_state
            caller = test_utils.find_test_caller()
            if caller:
                message = '(%s) %s' % (caller, message)
            raise lib_exc.TimeoutException(message)
        old_status = server_status
        old_task_state = task_state


def wait_for_server_termination(client, server_id, ignore_error=False,
                                request_id=None):
    """Waits for server to reach termination."""
    try:
        body = client.show_server(server_id)['server']
    except lib_exc.NotFound:
        return
    old_status = body['status']
    old_task_state = _get_task_state(body)
    start_time = int(time.time())
    while True:
        time.sleep(client.build_interval)
        try:
            body = client.show_server(server_id)['server']
        except lib_exc.NotFound:
            return
        server_status = body['status']
        task_state = _get_task_state(body)
        if (server_status != old_status) or (task_state != old_task_state):
            LOG.info('State transition "%s" ==> "%s" after %d second wait',
                     '/'.join((old_status, str(old_task_state))),
                     '/'.join((server_status, str(task_state))),
                     time.time() - start_time)
        if server_status == 'ERROR' and not ignore_error:
            details = ("Server %s failed to delete and is in ERROR status." %
                       server_id)
            if 'fault' in body:
                details += ' Fault: %s.' % body['fault']
            if request_id:
                details += ' Server delete request ID: %s.' % request_id
            raise lib_exc.DeleteErrorException(details, server_id=server_id)

        if server_status == 'SOFT_DELETED':
            # Soft-deleted instances need to be forcibly deleted to
            # prevent some test cases from failing.
            LOG.debug("Automatically force-deleting soft-deleted server %s",
                      server_id)
            try:
                client.force_delete_server(server_id)
            except lib_exc.NotFound:
                # The instance may have been deleted so ignore
                # NotFound exception
                return

        if int(time.time()) - start_time >= client.build_timeout:
            raise lib_exc.TimeoutException
        old_status = server_status
        old_task_state = task_state


def wait_for_image_status(client, image_id, status):
    """Waits for an image to reach a given status (or list of them).

    The client should have a show_image(image_id) method to get the image.
    The client should also have build_interval and build_timeout attributes.

    status can be either a string or a list of strings that constitute a
    terminal state that we will return.
    """
    show_image = client.show_image

    if isinstance(status, str):
        terminal_status = [status]
    else:
        terminal_status = status

    current_status = 'An unknown status'
    start = int(time.time())
    while int(time.time()) - start < client.build_timeout:
        image = show_image(image_id)
        # Compute image client returns response wrapped in 'image' element
        # which is not the case with Glance image client.
        if 'image' in image:
            image = image['image']

        current_status = image['status']
        if current_status in terminal_status:
            return current_status
        if current_status.lower() == 'killed':
            raise exceptions.ImageKilledException(image_id=image_id,
                                                  status=status)
        if current_status.lower() == 'error':
            raise exceptions.AddImageException(image_id=image_id)

        time.sleep(client.build_interval)

    message = ('Image %(image_id)s failed to reach %(status)s state '
               '(current state %(current_status)s) within the required '
               'time (%(timeout)s s).' % {'image_id': image_id,
                                          'status': ','.join(terminal_status),
                                          'current_status': current_status,
                                          'timeout': client.build_timeout})
    caller = test_utils.find_test_caller()
    if caller:
        message = '(%s) %s' % (caller, message)
    raise lib_exc.TimeoutException(message)


def wait_for_image_tasks_status(client, image_id, status):
    """Waits for an image tasks to reach a given status."""
    pending_tasks = []
    start = int(time.time())
    while int(time.time()) - start < client.build_timeout:
        tasks = client.show_image_tasks(image_id)['tasks']

        pending_tasks = [task for task in tasks if task['status'] != status]
        if not pending_tasks:
            return tasks
        time.sleep(client.build_interval)

    message = ('Image %(image_id)s tasks: %(pending_tasks)s '
               'failed to reach %(status)s state within the required '
               'time (%(timeout)s s).' % {'image_id': image_id,
                                          'pending_tasks': pending_tasks,
                                          'status': status,
                                          'timeout': client.build_timeout})
    caller = test_utils.find_test_caller()
    if caller:
        message = '(%s) %s' % (caller, message)
    raise lib_exc.TimeoutException(message)


def wait_for_tasks_status(client, task_id, status):
    start = int(time.time())
    while int(time.time()) - start < client.build_timeout:
        task = client.show_tasks(task_id)
        if task['status'] == status:
            return task
        time.sleep(client.build_interval)
    message = ('Task %(task_id)s tasks: '
               'failed to reach %(status)s state within the required '
               'time (%(timeout)s s).' % {'task_id': task_id,
                                          'status': status,
                                          'timeout': client.build_timeout})
    caller = test_utils.find_test_caller()
    if caller:
        message = '(%s) %s' % (caller, message)
    raise lib_exc.TimeoutException(message)


def wait_for_image_imported_to_stores(client, image_id, stores=None):
    """Waits for an image to be imported to all requested stores.

    Short circuits to fail if the serer reports failure of any store.
    If stores is None, just wait for status==active.

    The client should also have build_interval and build_timeout attributes.
    """

    exc_cls = lib_exc.TimeoutException
    start = int(time.time())

    # NOTE(danms): Don't wait for stores that are read-only as those
    # will never complete
    try:
        store_info = client.info_stores()['stores']
        stores = ','.join(sorted([
            store['id'] for store in store_info
            if store.get('read-only') != 'true' and
            (not stores or store['id'] in stores.split(','))]))
    except lib_exc.NotFound:
        # If multi-store is not enabled, then we can not resolve which
        # ones are read-only, and stores must have been passed as None
        # anyway for us to succeed. If not, then we should raise right
        # now and avoid waiting since we will never see the stores
        # appear.
        if stores is not None:
            raise lib_exc.TimeoutException(
                'Image service has no store support; '
                'cowardly refusing to wait for them.')

    while int(time.time()) - start < client.build_timeout:
        image = client.show_image(image_id)
        if image['status'] == 'active' and (stores is None or
                                            image['stores'] == stores):
            return
        if image.get('os_glance_failed_import'):
            exc_cls = lib_exc.OtherRestClientException
            break

        time.sleep(client.build_interval)

    message = ('Image %s failed to import on stores: %s' %
               (image_id, str(image.get('os_glance_failed_import'))))
    caller = test_utils.find_test_caller()
    if caller:
        message = '(%s) %s' % (caller, message)
    raise exc_cls(message)


def wait_for_image_copied_to_stores(client, image_id):
    """Waits for an image to be copied on all requested stores.

    The client should also have build_interval and build_timeout attributes.
    This return the list of stores where copy is failed.
    """

    start = int(time.time())
    store_left = []
    while int(time.time()) - start < client.build_timeout:
        image = client.show_image(image_id)
        store_left = image.get('os_glance_importing_to_stores')
        # NOTE(danms): If os_glance_importing_to_stores is None, then
        # we've raced with the startup of the task and should continue
        # to wait.
        if store_left is not None and not store_left:
            return image['os_glance_failed_import']
        if image['status'].lower() == 'killed':
            raise exceptions.ImageKilledException(image_id=image_id,
                                                  status=image['status'])

        time.sleep(client.build_interval)

    message = ('Image %s failed to finish the copy operation '
               'on stores: %s' % (image_id, str(store_left)))
    caller = test_utils.find_test_caller()
    if caller:
        message = '(%s) %s' % (caller, message)
    raise lib_exc.TimeoutException(message)


def wait_for_image_deleted_from_store(client, image, available_stores,
                                      image_store_deleted):
    """Waits for an image to be deleted from specific store.

    API will not allow deletion of the last location for an image.
    This return image if image deleted from store.
    """

    # Check if image have last store location
    if len(available_stores) == 1:
        exc_cls = lib_exc.OtherRestClientException
        message = 'Delete from last store location not allowed'
        raise exc_cls(message)
    start = int(time.time())
    while int(time.time()) - start < client.build_timeout:
        image = client.show_image(image['id'])
        image_stores = image['stores'].split(",")
        if image_store_deleted not in image_stores:
            return
        time.sleep(client.build_interval)
    message = ('Failed to delete %s from requested store location: %s '
               'within the required time: (%s s)' %
               (image, image_store_deleted, client.build_timeout))
    caller = test_utils.find_test_caller()
    if caller:
        message = '(%s) %s' % (caller, message)
    raise exc_cls(message)


def wait_for_volume_resource_status(client, resource_id, status,
                                    server_id=None, servers_client=None):
    """Waits for a volume resource to reach a given status.

    This function is a common function for volume, snapshot and backup
    resources. The function extracts the name of the desired resource from
    the client class name of the resource.

    If server_id and servers_client are provided, dump the console for that
    server on failure.
    """
    resource_name = re.findall(
        r'(volume|group-snapshot|snapshot|backup|group)',
        client.resource_type)[-1].replace('-', '_')
    show_resource = getattr(client, 'show_' + resource_name)
    resource_status = show_resource(resource_id)[resource_name]['status']
    start = int(time.time())

    while resource_status != status:
        time.sleep(client.build_interval)
        resource_status = show_resource(resource_id)[
            '{}'.format(resource_name)]['status']
        if resource_status == 'error' and resource_status != status:
            raise exceptions.VolumeResourceBuildErrorException(
                resource_name=resource_name, resource_id=resource_id)
        if resource_name == 'volume' and resource_status == 'error_restoring':
            raise exceptions.VolumeRestoreErrorException(volume_id=resource_id)
        if resource_status == 'error_extending' and resource_status != status:
            raise exceptions.VolumeExtendErrorException(volume_id=resource_id)

        if int(time.time()) - start >= client.build_timeout:
            if server_id and servers_client:
                console_output = servers_client.get_console_output(
                    server_id)['output']
                LOG.debug('Console output for %s\nbody=\n%s',
                          server_id, console_output)
            message = ('%s %s failed to reach %s status (current %s) '
                       'within the required time (%s s).' %
                       (resource_name, resource_id, status, resource_status,
                        client.build_timeout))
            raise lib_exc.TimeoutException(message)
    LOG.info('%s %s reached %s after waiting for %f seconds',
             resource_name, resource_id, status, time.time() - start)


def wait_for_volume_attachment_create(client, volume_id, server_id):
    """Waits for a volume attachment to be created at a given volume."""
    start = int(time.time())
    while True:
        attachments = client.show_volume(volume_id)['volume']['attachments']
        found = [a for a in attachments if a['server_id'] == server_id]
        if found:
            LOG.info('Attachment %s created for volume %s to server %s after '
                     'waiting for %f seconds', found[0]['attachment_id'],
                     volume_id, server_id, time.time() - start)
            return found[0]
        time.sleep(client.build_interval)
        if int(time.time()) - start >= client.build_timeout:
            message = ('Failed to attach volume %s to server %s '
                       'within the required time (%s s).' %
                       (volume_id, server_id, client.build_timeout))
            raise lib_exc.TimeoutException(message)


def wait_for_volume_attachment_remove(client, volume_id, attachment_id):
    """Waits for a volume attachment to be removed from a given volume."""
    start = int(time.time())
    attachments = client.show_volume(volume_id)['volume']['attachments']
    while any(attachment_id == a['attachment_id'] for a in attachments):
        time.sleep(client.build_interval)
        if int(time.time()) - start >= client.build_timeout:
            message = ('Failed to remove attachment %s from volume %s '
                       'within the required time (%s s).' %
                       (attachment_id, volume_id, client.build_timeout))
            raise lib_exc.TimeoutException(message)
        attachments = client.show_volume(volume_id)['volume']['attachments']
    LOG.info('Attachment %s removed from volume %s after waiting for %f '
             'seconds', attachment_id, volume_id, time.time() - start)


def wait_for_volume_attachment_remove_from_server(
        client, server_id, volume_id):
    """Waits for a volume to be removed from a given server.

    This waiter checks the compute API if the volume attachment is removed.
    """
    start = int(time.time())

    try:
        volumes = client.list_volume_attachments(
            server_id)['volumeAttachments']
    except lib_exc.NotFound:
        # Ignore 404s on detach in case the server is deleted or the volume
        # is already detached.
        return

    while any(volume for volume in volumes if volume['volumeId'] == volume_id):
        time.sleep(client.build_interval)

        timed_out = int(time.time()) - start >= client.build_timeout
        if timed_out:
            console_output = client.get_console_output(server_id)['output']
            LOG.debug('Console output for %s\nbody=\n%s',
                      server_id, console_output)
            message = ('Volume %s failed to detach from server %s within '
                       'the required time (%s s) from the compute API '
                       'perspective' %
                       (volume_id, server_id, client.build_timeout))
            raise lib_exc.TimeoutException(message)
        try:
            volumes = client.list_volume_attachments(
                server_id)['volumeAttachments']
        except lib_exc.NotFound:
            # Ignore 404s on detach in case the server is deleted or the volume
            # is already detached.
            return
    return


def wait_for_volume_migration(client, volume_id, new_host):
    """Waits for a Volume to move to a new host."""
    body = client.show_volume(volume_id)['volume']
    host = body['os-vol-host-attr:host']
    migration_status = body['migration_status']
    start = int(time.time())

    # new_host is hostname@backend while current_host is hostname@backend#type
    while migration_status != 'success' or new_host not in host:
        time.sleep(client.build_interval)
        body = client.show_volume(volume_id)['volume']
        host = body['os-vol-host-attr:host']
        migration_status = body['migration_status']

        if migration_status == 'error':
            message = ('volume %s failed to migrate.' % (volume_id))
            raise lib_exc.TempestException(message)

        if int(time.time()) - start >= client.build_timeout:
            message = ('Volume %s failed to migrate to %s (current %s) '
                       'within the required time (%s s).' %
                       (volume_id, new_host, host, client.build_timeout))
            raise lib_exc.TimeoutException(message)


def wait_for_volume_retype(client, volume_id, new_volume_type):
    """Waits for a Volume to have a new volume type."""
    body = client.show_volume(volume_id)['volume']
    current_volume_type = body['volume_type']
    start = int(time.time())

    while current_volume_type != new_volume_type:
        time.sleep(client.build_interval)
        body = client.show_volume(volume_id)['volume']
        current_volume_type = body['volume_type']

        if int(time.time()) - start >= client.build_timeout:
            message = ('Volume %s failed to reach %s volume type (current %s) '
                       'within the required time (%s s).' %
                       (volume_id, new_volume_type, current_volume_type,
                        client.build_timeout))
            raise lib_exc.TimeoutException(message)


def wait_for_qos_operations(client, qos_id, operation, args=None):
    """Waits for a qos operations to be completed.

    NOTE : operation value is required for  wait_for_qos_operations()
    operation = 'qos-key' / 'disassociate' / 'disassociate-all'
    args = keys[] when operation = 'qos-key'
    args = volume-type-id disassociated when operation = 'disassociate'
    args = None when operation = 'disassociate-all'
    """
    start_time = int(time.time())
    while True:
        if operation == 'qos-key-unset':
            body = client.show_qos(qos_id)['qos_specs']
            if not any(key in body['specs'] for key in args):
                return
        elif operation == 'disassociate':
            body = client.show_association_qos(qos_id)['qos_associations']
            if not any(args in body[i]['id'] for i in range(0, len(body))):
                return
        elif operation == 'disassociate-all':
            body = client.show_association_qos(qos_id)['qos_associations']
            if not body:
                return
        else:
            msg = (" operation value is either not defined or incorrect.")
            raise lib_exc.UnprocessableEntity(msg)

        if int(time.time()) - start_time >= client.build_timeout:
            raise lib_exc.TimeoutException
        time.sleep(client.build_interval)


def wait_for_interface_status(client, server_id, port_id, status):
    """Waits for an interface to reach a given status."""
    body = (client.show_interface(server_id, port_id)
            ['interfaceAttachment'])
    interface_status = body['port_state']
    start = int(time.time())

    while interface_status != status:
        time.sleep(client.build_interval)
        body = (client.show_interface(server_id, port_id)
                ['interfaceAttachment'])
        interface_status = body['port_state']

        timed_out = int(time.time()) - start >= client.build_timeout

        if interface_status != status and timed_out:
            message = ('Interface %s failed to reach %s status '
                       '(current %s) within the required time (%s s).' %
                       (port_id, status, interface_status,
                        client.build_timeout))
            raise lib_exc.TimeoutException(message)

    return body


def wait_for_interface_detach(client, server_id, port_id, detach_request_id):
    """Waits for an interface to be detached from a server."""
    def _get_detach_event_results():
        # NOTE(gibi): The obvious choice for this waiter would be to wait
        # until the interface disappears from the client.list_interfaces()
        # response. However that response is based on the binding status of the
        # port in Neutron. Nova deallocates the port resources _after the port
        # was  unbound in Neutron. This can cause that the naive waiter would
        # return before the port is fully deallocated. Wait instead of the
        # os-instance-action to succeed as that is recorded after both the
        # port is fully deallocated.
        events = client.show_instance_action(
            server_id, detach_request_id)['instanceAction'].get('events', [])
        return [
            event['result'] for event in events
            if event['event'] == 'compute_detach_interface'
        ]

    detach_event_results = _get_detach_event_results()

    start = int(time.time())

    while "Success" not in detach_event_results:
        time.sleep(client.build_interval)
        detach_event_results = _get_detach_event_results()
        if "Success" in detach_event_results:
            return client.show_instance_action(
                server_id, detach_request_id)['instanceAction']

        timed_out = int(time.time()) - start >= client.build_timeout
        if timed_out:
            message = ('Interface %s failed to detach from server %s within '
                       'the required time (%s s)' % (port_id, server_id,
                                                     client.build_timeout))
            raise lib_exc.TimeoutException(message)


def wait_for_server_floating_ip(servers_client, server, floating_ip,
                                wait_for_disassociate=False):
    """Wait for floating IP association or disassociation.

    :param servers_client: The servers client to use when querying the server's
    floating IPs.
    :param server: The server JSON dict on which to wait.
    :param floating_ip: The floating IP JSON dict on which to wait.
    :param wait_for_disassociate: Boolean indicating whether to wait for
    disassociation instead of association.
    """

    def _get_floating_ip_in_server_addresses(floating_ip, server):
        for addresses in server['addresses'].values():
            for address in addresses:
                if (
                    address['OS-EXT-IPS:type'] == 'floating' and
                    address['addr'] == floating_ip['floating_ip_address']
                ):
                    return address
        return None

    start_time = int(time.time())
    while True:
        server = servers_client.show_server(server['id'])['server']
        address = _get_floating_ip_in_server_addresses(floating_ip, server)
        if address is None and wait_for_disassociate:
            return None
        if not wait_for_disassociate and address:
            return address

        if int(time.time()) - start_time >= servers_client.build_timeout:
            if wait_for_disassociate:
                msg = ('Floating ip %s failed to disassociate from server %s '
                       'in time.' % (floating_ip, server['id']))
            else:
                msg = ('Floating ip %s failed to associate with server %s '
                       'in time.' % (floating_ip, server['id']))
            raise lib_exc.TimeoutException(msg)
        time.sleep(servers_client.build_interval)


def wait_for_ping(server_ip, timeout=30, interval=1):
    """Waits for an address to become pingable"""
    start_time = int(time.time())
    while int(time.time()) - start_time < timeout:
        response = os.system("ping -c 1 " + server_ip)
        if response == 0:
            return
        time.sleep(interval)
    raise lib_exc.TimeoutException()


def wait_for_port_status(client, port_id, status):
    """Wait for a port reach a certain status : ["BUILD" | "DOWN" | "ACTIVE"]
    :param client: The network client to use when querying the port's
    status
    :param status: A string to compare the current port status-to.
    :param port_id: The uuid of the port we would like queried for status.
    """
    start_time = time.time()
    while (time.time() - start_time <= client.build_timeout):
        result = client.show_port(port_id)
        if result['port']['status'].lower() == status.lower():
            return result
        time.sleep(client.build_interval)
    raise lib_exc.TimeoutException


def wait_for_server_ports_active(client, server_id, is_active, **kwargs):
    """Wait for all server ports to reach active status
    :param client: The network client to use when querying the port's status
    :param server_id: The uuid of the server's ports we need to verify.
    :param is_active: A function to call to the check port active status.
    :param kwargs: Additional arguments, if any, to pass to list_ports()
    """
    start_time = time.time()
    while (time.time() - start_time <= client.build_timeout):
        ports = client.list_ports(device_id=server_id, **kwargs)['ports']
        if all(is_active(port) for port in ports):
            LOG.debug("Server ID %s ports are all ACTIVE %s: ",
                      server_id, ports)
            return ports
        LOG.warning("Server ID %s has ports that are not ACTIVE, waiting "
                    "for state to change on all: %s", server_id, ports)
        time.sleep(client.build_interval)
    LOG.error("Server ID %s ports have failed to transition to ACTIVE, "
              "timing out: %s", server_id, ports)
    raise lib_exc.TimeoutException


def wait_for_ssh(ssh_client, timeout=30):
    """Waits for SSH connection to become usable"""
    start_time = int(time.time())
    while int(time.time()) - start_time < timeout:
        try:
            ssh_client.validate_authentication()
            return
        except lib_exc.SSHTimeout:
            pass
    raise lib_exc.TimeoutException()


def wait_for_caching(client, cache_client, image_id):
    """Waits until image is cached"""
    start = int(time.time())
    while int(time.time()) - start < client.build_timeout:
        caching = cache_client.list_cache()
        output = [image['image_id'] for image in caching['cached_images']]
        if output and image_id in output:
            return caching

        time.sleep(client.build_interval)

    message = ('Image %s failed to cache in time.' % image_id)
    caller = test_utils.find_test_caller()
    if caller:
        message = '(%s) %s' % (caller, message)
    raise lib_exc.TimeoutException(message)


def wait_for_object_create(object_client, container_name, object_name,
                           interval=1):
    """Waits for created object to become available"""
    start_time = time.time()
    while time.time() - start_time < object_client.build_timeout:
        try:
            return object_client.get_object(container_name, object_name)
        except lib_exc.NotFound:
            time.sleep(interval)
    message = ('Object %s failed to create within the required time (%s s).' %
               (object_name, object_client.build_timeout))
    raise lib_exc.TimeoutException(message)



# Copyright 2023 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions
from tempest.scenario import manager

CONF = config.CONF


class BaseAttachmentTest(manager.ScenarioTest):

    @classmethod
    def skip_checks(cls):
        super(BaseAttachmentTest, cls).skip_checks()
        if not CONF.service_available.cinder:
            raise cls.skipException("Cinder is not available")

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.attachments_client = cls.os_primary.attachments_client_latest
        cls.admin_volume_client = cls.os_admin.volumes_client_latest

    def _call_with_fake_service_token(self, valid_token,
                                      client, method_name, *args, **kwargs):
        """Call client method with non-service service token

        Add a service token header that can be a valid normal user token (which
        won't have the service role) or an invalid token altogether.
        """
        original_raw_request = client.raw_request

        def raw_request(url, method, headers=None, body=None, chunked=False,
                        log_req_body=None):
            token = headers['X-Auth-Token']
            if not valid_token:
                token = token[:-1] + ('a' if token[-1] != 'a' else 'b')
            headers['X-Service-Token'] = token
            return original_raw_request(url, method, headers=headers,
                                        body=body, chunked=chunked,
                                        log_req_body=log_req_body)

        client_method = getattr(client, method_name)
        with mock.patch.object(client, 'raw_request', raw_request):
            return client_method(*args, **kwargs)


class TestServerVolumeAttachmentScenario(BaseAttachmentTest):

    """Test server attachment behaviors

    This tests that volume attachments to servers may not be removed directly
    and are only allowed through the compute service (bug #2004555).
    """

    @decorators.attr(type='slow')
    @decorators.idempotent_id('be615530-f105-437a-8afe-ce998c9535d9')
    @utils.services('compute', 'volume', 'image', 'network')
    def test_server_detach_rules(self):
        """Test that various methods of detaching a volume honors the rules"""
        volume = self.create_volume(wait_until=None)
        volume2 = self.create_volume(wait_until=None)

        server = self.create_server(wait_until='SSHABLE')
        servers = self.servers_client.list_servers()['servers']
        self.assertIn(server['id'], [x['id'] for x in servers])

        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        # The volume retrieved on creation has a non-up-to-date status.
        # Retrieval after it becomes active ensures correct details.
        volume = self.volumes_client.show_volume(volume['id'])['volume']

        volume = self.nova_volume_attach(server, volume)
        self.addCleanup(self.nova_volume_detach, server, volume)
        att_id = volume['attachments'][0]['attachment_id']

        # Test user call to detach volume is rejected
        self.assertRaises((exceptions.Forbidden, exceptions.Conflict),
                          self.volumes_client.detach_volume, volume['id'])

        # Test user call to terminate connection is rejected
        self.assertRaises((exceptions.Forbidden, exceptions.Conflict),
                          self.volumes_client.terminate_connection,
                          volume['id'], connector={})

        # Test faking of service token on call to detach, force detach,
        # terminate_connection
        for valid_token in (True, False):
            valid_exceptions = [exceptions.Forbidden, exceptions.Conflict]
            if not valid_token:
                valid_exceptions.append(exceptions.Unauthorized)
            self.assertRaises(
                tuple(valid_exceptions),
                self._call_with_fake_service_token,
                valid_token,
                self.volumes_client,
                'detach_volume',
                volume['id'])
            self.assertRaises(
                tuple(valid_exceptions),
                self._call_with_fake_service_token,
                valid_token,
                self.volumes_client,
                'terminate_connection',
                volume['id'], connector={})

        # Reset volume's status to error
        self.admin_volume_client.reset_volume_status(volume['id'],
                                                     status='error')
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'error')

        # For the cleanup, we need to reset the volume status to in-use before
        # the other cleanup steps try to detach it.
        self.addCleanup(waiters.wait_for_volume_resource_status,
                        self.volumes_client, volume['id'], 'in-use')
        self.addCleanup(self.admin_volume_client.reset_volume_status,
                        volume['id'], status='in-use')

        # Test user call to force detach volume is rejected
        self.assertRaises(
            (exceptions.Forbidden, exceptions.Conflict),
            self.admin_volume_client.force_detach_volume,
            volume['id'], connector=None,
            attachment_id=att_id)

        # Test trying to override detach with force and service token
        for valid_token in (True, False):
            valid_exceptions = [exceptions.Forbidden, exceptions.Conflict]
            if not valid_token:
                valid_exceptions.append(exceptions.Unauthorized)
            self.assertRaises(
                tuple(valid_exceptions),
                self._call_with_fake_service_token,
                valid_token,
                self.admin_volume_client,
                'force_detach_volume',
                volume['id'], connector=None, attachment_id=att_id)

        # Test user call to detach with mismatch is rejected
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume2['id'], 'available')
        # The volume retrieved on creation has a non-up-to-date status.
        # Retrieval after it becomes active ensures correct details.
        volume2 = self.volumes_client.show_volume(volume2['id'])['volume']

        volume2 = self.nova_volume_attach(server, volume2)
        att_id2 = volume2['attachments'][0]['attachment_id']
        self.assertRaises(
            (exceptions.Forbidden, exceptions.BadRequest),
            self.volumes_client.detach_volume,
            volume['id'], attachment_id=att_id2)


class TestServerVolumeAttachScenarioOldVersion(BaseAttachmentTest):
    volume_min_microversion = '3.27'
    volume_max_microversion = 'latest'

    @decorators.attr(type='slow')
    @decorators.idempotent_id('6f4d2144-99f4-495c-8b0b-c6a537971418')
    @utils.services('compute', 'volume', 'image', 'network')
    def test_old_versions_reject(self):
        server = self.create_server(wait_until='SSHABLE')
        servers = self.servers_client.list_servers()['servers']
        self.assertIn(server['id'], [x['id'] for x in servers])

        volume = self.create_volume()

        volume = self.nova_volume_attach(server, volume)
        self.addCleanup(self.nova_volume_detach, server, volume)
        att_id = volume['attachments'][0]['attachment_id']

        for valid_token in (True, False):
            valid_exceptions = [exceptions.Forbidden,
                                exceptions.Conflict]
            if not valid_token:
                valid_exceptions.append(exceptions.Unauthorized)
            self.assertRaises(
                tuple(valid_exceptions),
                self._call_with_fake_service_token,
                valid_token,
                self.attachments_client,
                'delete_attachment',
                att_id)

        self.assertRaises(
            (exceptions.Forbidden, exceptions.Conflict),
            self.attachments_client.delete_attachment,
            att_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.network import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

CONF = config.CONF


class ExternalNetworksTestJSON(base.BaseAdminNetworkTest):
    """Test external networks"""

    @classmethod
    def resource_setup(cls):
        super(ExternalNetworksTestJSON, cls).resource_setup()
        cls.network = cls.create_network()

    def _create_network(self, external=True):
        post_body = {'name': data_utils.rand_name(
            name='network-', prefix=CONF.resource_name_prefix)}
        if external:
            post_body['router:external'] = external
        body = self.admin_networks_client.create_network(**post_body)
        network = body['network']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.admin_networks_client.delete_network, network['id'])
        return network

    @decorators.idempotent_id('462be770-b310-4df9-9c42-773217e4c8b1')
    def test_create_external_network(self):
        """Test creating external network

        Create a network as an admin user specifying the
        external network extension attribute
        """
        ext_network = self._create_network()
        # Verifies router:external parameter
        self.assertIsNotNone(ext_network['id'])
        self.assertTrue(ext_network['router:external'])

    @decorators.idempotent_id('4db5417a-e11c-474d-a361-af00ebef57c5')
    def test_update_external_network(self):
        """Test updating external network

        Update a network as an admin user specifying the
        external network extension attribute
        """
        network = self._create_network(external=False)
        self.assertFalse(network.get('router:external', False))
        update_body = {'router:external': True}
        body = self.admin_networks_client.update_network(network['id'],
                                                         **update_body)
        updated_network = body['network']
        # Verify that router:external parameter was updated
        self.assertTrue(updated_network['router:external'])

    @decorators.idempotent_id('39be4c9b-a57e-4ff9-b7c7-b218e209dfcc')
    def test_list_external_networks(self):
        """Test listing external networks"""
        # Create external_net
        external_network = self._create_network()
        # List networks as a normal user and confirm the external
        # network extension attribute is returned for those networks
        # that were created as external
        body = self.networks_client.list_networks()
        networks_list = [net['id'] for net in body['networks']]
        self.assertIn(external_network['id'], networks_list)
        self.assertIn(self.network['id'], networks_list)
        for net in body['networks']:
            if net['id'] == self.network['id']:
                self.assertFalse(net['router:external'])
            elif net['id'] == external_network['id']:
                self.assertTrue(net['router:external'])

    @decorators.idempotent_id('2ac50ab2-7ebd-4e27-b3ce-a9e399faaea2')
    def test_show_external_networks_attribute(self):
        """Test showing external network attribute"""
        # Create external_net
        external_network = self._create_network()
        # Show an external network as a normal user and confirm the
        # external network extension attribute is returned.
        body = self.networks_client.show_network(external_network['id'])
        show_ext_net = body['network']
        self.assertEqual(external_network['name'], show_ext_net['name'])
        self.assertEqual(external_network['id'], show_ext_net['id'])
        self.assertTrue(show_ext_net['router:external'])
        body = self.networks_client.show_network(self.network['id'])
        show_net = body['network']
        # Verify with show that router:external is False for network
        self.assertEqual(self.network['name'], show_net['name'])
        self.assertEqual(self.network['id'], show_net['id'])
        self.assertFalse(show_net['router:external'])

    @decorators.idempotent_id('82068503-2cf2-4ed4-b3be-ecb89432e4bb')
    @testtools.skipUnless(CONF.network_feature_enabled.floating_ips,
                          'Floating ips are not availabled')
    def test_delete_external_networks_with_floating_ip(self):
        """Test deleting external network with unassociated floating ips

        Verifies external network can be deleted while still holding
        (unassociated) floating IPs
        """
        body = self.admin_networks_client.create_network(
            **{'router:external': True})
        external_network = body['network']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_networks_client.delete_network,
                        external_network['id'])
        subnet = self.create_subnet(
            external_network, client=self.admin_subnets_client,
            enable_dhcp=False)
        body = self.admin_floating_ips_client.create_floatingip(
            floating_network_id=external_network['id'])
        created_floating_ip = body['floatingip']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_floating_ips_client.delete_floatingip,
                        created_floating_ip['id'])
        if utils.is_extension_enabled('filter-validation', 'network'):
            floatingip_list = self.admin_floating_ips_client.list_floatingips(
                floating_network_id=external_network['id'])
        else:
            # NOTE(hongbin): This is for testing the backward-compatibility
            # of neutron API although the parameter is a wrong filter
            # for listing floating IPs.
            floatingip_list = self.admin_floating_ips_client.list_floatingips(
                invalid_filter=external_network['id'])
        self.assertIn(created_floating_ip['id'],
                      (f['id'] for f in floatingip_list['floatingips']))
        self.admin_networks_client.delete_network(external_network['id'])
        # Verifies floating ip is deleted
        floatingip_list = self.admin_floating_ips_client.list_floatingips()
        self.assertNotIn(created_floating_ip['id'],
                         (f['id'] for f in floatingip_list['floatingips']))
        # Verifies subnet is deleted
        subnet_list = self.admin_subnets_client.list_subnets()
        self.assertNotIn(subnet['id'],
                         (s['id'] for s in subnet_list))



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import urllib3

from tempest.lib.common import http
from tempest.tests import base


CERT_NONE = 'CERT_NONE'
CERT_REQUIRED = 'CERT_REQUIRED'
CERT_LOCATION = '/etc/ssl/certs/ca-certificates.crt'
PROXY_URL = 'http://myproxy:3128'
REQUEST_URL = 'http://10.0.0.107:5000/v2.0'
REQUEST_METHOD = 'GET'


class TestClosingHttp(base.TestCase):

    def closing_http(self, **kwargs):
        return http.ClosingHttp(**kwargs)

    def test_closing_http(self):
        connection = self.closing_http()

        self.assertNotIn('cert_reqs', connection.connection_pool_kw)
        self.assertNotIn('ca_certs', connection.connection_pool_kw)
        self.assertNotIn('timeout', connection.connection_pool_kw)

    def test_closing_http_with_ca_certs(self):
        connection = self.closing_http(ca_certs=CERT_LOCATION)

        self.assertEqual(CERT_REQUIRED,
                         connection.connection_pool_kw['cert_reqs'])
        self.assertEqual(CERT_LOCATION,
                         connection.connection_pool_kw['ca_certs'])

    def test_closing_http_with_dscv(self):
        connection = self.closing_http(
            disable_ssl_certificate_validation=True)

        self.assertEqual(CERT_NONE,
                         connection.connection_pool_kw['cert_reqs'])
        self.assertNotIn('ca_certs',
                         connection.connection_pool_kw)

    def test_closing_http_with_ca_certs_and_dscv(self):
        connection = self.closing_http(
            disable_ssl_certificate_validation=True,
            ca_certs=CERT_LOCATION)

        self.assertEqual(CERT_NONE,
                         connection.connection_pool_kw['cert_reqs'])
        self.assertNotIn('ca_certs',
                         connection.connection_pool_kw)

    def test_closing_http_with_timeout(self):
        timeout = 30
        connection = self.closing_http(timeout=timeout)
        self.assertEqual(timeout,
                         connection.connection_pool_kw['timeout'])

    def test_request(self):
        # Given
        connection = self.closing_http()
        http_response = urllib3.HTTPResponse()
        request = self.patch('urllib3.PoolManager.request',
                             return_value=http_response)
        retry = self.patch('urllib3.util.Retry')

        # When
        response, data = connection.request(
            method=REQUEST_METHOD,
            url=REQUEST_URL)

        # Then
        request.assert_called_once_with(
            REQUEST_METHOD,
            REQUEST_URL,
            headers={'connection': 'close'},
            retries=retry(raise_on_redirect=False, redirect=5))
        self.assertEqual(
            {'content-location': REQUEST_URL,
             'status': str(http_response.status)},
            response)
        self.assertEqual(http_response.status, response.status)
        self.assertEqual(http_response.reason, response.reason)
        self.assertEqual(http_response.version, response.version)
        self.assertEqual(http_response.data, data)

    def test_request_with_fields(self):
        # Given
        connection = self.closing_http()
        http_response = urllib3.HTTPResponse()
        request = self.patch('urllib3.PoolManager.request',
                             return_value=http_response)
        retry = self.patch('urllib3.util.Retry')
        fields = object()

        # When
        connection.request(
            method=REQUEST_METHOD,
            url=REQUEST_URL,
            fields=fields)

        # Then
        request.assert_called_once_with(
            REQUEST_METHOD,
            REQUEST_URL,
            fields=fields,
            headers=dict(connection='close'),
            retries=retry(raise_on_redirect=False, redirect=5))

    def test_request_with_headers(self):
        # Given
        connection = self.closing_http()
        headers = {'Xtra Key': 'Xtra Value'}
        http_response = urllib3.HTTPResponse(headers=headers)
        request = self.patch('urllib3.PoolManager.request',
                             return_value=http_response)
        retry = self.patch('urllib3.util.Retry')

        # When
        response, _ = connection.request(
            method=REQUEST_METHOD,
            url=REQUEST_URL,
            headers=headers)

        # Then
        request.assert_called_once_with(
            REQUEST_METHOD,
            REQUEST_URL,
            headers=dict(headers, connection='close'),
            retries=retry(raise_on_redirect=False, redirect=5))
        self.assertEqual(
            {'content-location': REQUEST_URL,
             'status': str(http_response.status),
             'xtra key': 'Xtra Value'},
            response)

    def test_request_preload(self):
        # Given
        connection = self.closing_http()
        headers = {'Xtra Key': 'Xtra Value'}
        http_response = urllib3.HTTPResponse(headers=headers)
        request = self.patch('urllib3.PoolManager.request',
                             return_value=http_response)
        retry = self.patch('urllib3.util.Retry')

        # When
        response, _ = connection.request(
            method=REQUEST_METHOD,
            url=REQUEST_URL,
            headers=headers,
            preload_content=False)

        # Then
        request.assert_called_once_with(
            REQUEST_METHOD,
            REQUEST_URL,
            headers=dict(headers, connection='close'),
            preload_content=False,
            retries=retry(raise_on_redirect=False, redirect=5))
        self.assertIsInstance(response, urllib3.HTTPResponse)


class TestClosingProxyHttp(TestClosingHttp):

    def closing_http(self, proxy_url=PROXY_URL, **kwargs):
        connection = http.ClosingProxyHttp(proxy_url=proxy_url, **kwargs)
        self.assertHasProxy(connection, proxy_url)
        return connection

    def test_class_without_proxy_url(self):
        self.assertRaises(ValueError, http.ClosingProxyHttp, None)

    def assertHasProxy(self, connection, proxy_url):
        self.assertIsInstance(connection, http.ClosingProxyHttp)
        proxy = connection.proxy
        self.assertEqual(proxy_url,
                         '%s://%s:%i' % (proxy.scheme,
                                         proxy.host,
                                         proxy.port))


class TestClosingHttpRedirects(base.TestCase):
    def test_redirect_default(self):
        connection = http.ClosingHttp()
        self.assertTrue(connection.follow_redirects)

    def test_redirect_off(self):
        connection = http.ClosingHttp(follow_redirects=False)
        self.assertFalse(connection.follow_redirects)


class TestClosingProxyHttpRedirects(base.TestCase):
    def test_redirect_default(self):
        connection = http.ClosingProxyHttp(proxy_url=PROXY_URL)
        self.assertTrue(connection.follow_redirects)

    def test_redirect_off(self):
        connection = http.ClosingProxyHttp(follow_redirects=False,
                                           proxy_url=PROXY_URL)
        self.assertFalse(connection.follow_redirects)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class VolumeMultiBackendTest(base.BaseVolumeAdminTest):
    """Test volume multi backends"""

    @classmethod
    def skip_checks(cls):
        super(VolumeMultiBackendTest, cls).skip_checks()

        if not CONF.volume_feature_enabled.multi_backend:
            raise cls.skipException("Cinder multi-backend feature disabled")

        if len(set(CONF.volume.backend_names)) < 2:
            raise cls.skipException("Requires at least two different "
                                    "backend names")

    @classmethod
    def resource_setup(cls):
        super(VolumeMultiBackendTest, cls).resource_setup()

        # read backend name from a list .
        backend_names = set(CONF.volume.backend_names)

        cls.volume_id_list_with_prefix = []
        cls.volume_id_list_without_prefix = []

        # Volume/Type creation (uses volume_backend_name)
        # It is not allowed to create the same backend name twice
        for backend_name in backend_names:
            # Volume/Type creation (uses backend_name)
            cls._create_type_and_volume(backend_name, False)
            # Volume/Type creation (uses capabilities:volume_backend_name)
            cls._create_type_and_volume(backend_name, True)

    @classmethod
    def _create_type_and_volume(cls, backend_name_key, with_prefix):
        # Volume/Type creation
        type_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=cls.__name__ + '-Type')
        vol_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=cls.__name__ + '-Volume')
        spec_key_with_prefix = "capabilities:volume_backend_name"
        spec_key_without_prefix = "volume_backend_name"
        if with_prefix:
            extra_specs = {spec_key_with_prefix: backend_name_key}
        else:
            extra_specs = {spec_key_without_prefix: backend_name_key}
        cls.create_volume_type(name=type_name,
                               extra_specs=extra_specs)

        params = {'name': vol_name, 'volume_type': type_name,
                  'size': CONF.volume.volume_size}
        cls.volume = cls.create_volume(**params)
        if with_prefix:
            cls.volume_id_list_with_prefix.append(cls.volume['id'])
        else:
            cls.volume_id_list_without_prefix.append(
                cls.volume['id'])
        waiters.wait_for_volume_resource_status(cls.admin_volume_client,
                                                cls.volume['id'], 'available')

    @decorators.idempotent_id('c1a41f3f-9dad-493e-9f09-3ff197d477cc')
    def test_backend_name_reporting(self):
        """Test backend name reporting for volume when type is without prefix

        1. Create volume type, with 'volume_backend_name' as extra spec key
        2. Create volume using the created volume type
        3. Check 'os-vol-host-attr:host' of the volume info, the value should
           contain '@' character, like 'cinder@CloveStorage#tecs_backend'
        """
        for volume_id in self.volume_id_list_without_prefix:
            self._test_backend_name_reporting_by_volume_id(volume_id)

    @decorators.idempotent_id('f38e647f-ab42-4a31-a2e7-ca86a6485215')
    def test_backend_name_reporting_with_prefix(self):
        """Test backend name reporting for volume when type is with prefix

        1. Create volume type, with 'capabilities:volume_backend_name' as
           extra spec key
        2. Create volume using the created volume type
        3. Check 'os-vol-host-attr:host' of the volume info, the value should
           contain '@' character, like 'cinder@CloveStorage#tecs_backend'
        """
        for volume_id in self.volume_id_list_with_prefix:
            self._test_backend_name_reporting_by_volume_id(volume_id)

    @decorators.idempotent_id('46435ab1-a0af-4401-8373-f14e66b0dd58')
    def test_backend_name_distinction(self):
        """Test volume backend distinction when type is without prefix

        1. For each backend, create volume type with 'volume_backend_name'
           as extra spec key
        2. Create volumes using the created volume types
        3. Check 'os-vol-host-attr:host' of each created volume is different.
        """
        self._test_backend_name_distinction(self.volume_id_list_without_prefix)

    @decorators.idempotent_id('4236305b-b65a-4bfc-a9d2-69cb5b2bf2ed')
    def test_backend_name_distinction_with_prefix(self):
        """Test volume backend distinction when type is with prefix

        1. For each backend, create volume type with
           'capabilities:volume_backend_name' as extra spec key
        2. Create volumes using the created volume types
        3. Check 'os-vol-host-attr:host' of each created volume is different.
        """
        self._test_backend_name_distinction(self.volume_id_list_with_prefix)

    def _get_volume_host(self, volume_id):
        return self.admin_volume_client.show_volume(
            volume_id)['volume']['os-vol-host-attr:host']

    def _test_backend_name_reporting_by_volume_id(self, volume_id):
        # this test checks if os-vol-attr:host is populated correctly after
        # the multi backend feature has been enabled
        # if multi-backend is enabled: os-vol-attr:host should be like:
        # host@backend_name
        volume = self.admin_volume_client.show_volume(volume_id)['volume']

        volume1_host = volume['os-vol-host-attr:host']
        msg = ("multi-backend reporting incorrect values for volume %s" %
               volume_id)
        self.assertGreater(len(volume1_host.split("@")), 1, msg)

    def _test_backend_name_distinction(self, volume_id_list):
        # this test checks that the volumes created at setUp don't
        # belong to the same backend (if they are, than the
        # volume backend distinction is not working properly)
        volume_hosts = [self._get_volume_host(volume) for volume in
                        volume_id_list]
        # assert that volumes are each created on separate hosts:
        msg = ("volumes %s were created in the same backend" % ", "
               .join(volume_hosts))
        self.assertCountEqual(volume_hosts, set(volume_hosts), msg)



# Copyright 2016 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import routers_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestRoutersClient(base.BaseServiceTest):
    FAKE_CREATE_ROUTER = {
        "router": {
            "admin_state_up": True,
            "availability_zone_hints": [],
            "availability_zones": [
                "nova"
            ],
            "created_at": "2018-03-19T19:17:04Z",
            "description": "",
            "distributed": False,
            "external_gateway_info": {
                "enable_snat": True,
                "external_fixed_ips": [
                    {
                        "ip_address": "172.24.4.6",
                        "subnet_id": "b930d7f6-ceb7-40a0-8b81-a425dd994ccf"
                    }
                ],
                "network_id": "ae34051f-aa6c-4c75-abf5-50dc9ac99ef3"
            },
            "flavor_id": "f7b14d9a-b0dc-4fbe-bb14-a0f4970a69e0",
            "ha": False,
            "id": "f8a44de0-fc8e-45df-93c7-f79bf3b01c95",
            "name": "router1",
            "routes": [],
            "revision_number": 1,
            "status": "ACTIVE",
            "updated_at": "2018-03-19T19:17:22Z",
            "project_id": "0bd18306d801447bb457a46252d82d13",
            "tenant_id": "0bd18306d801447bb457a46252d82d13",
            "service_type_id": None,
            "tags": ["tag1,tag2"],
            "conntrack_helpers": []
        }
    }

    FAKE_UPDATE_ROUTER = {
        "router": {
            "admin_state_up": True,
            "availability_zone_hints": [],
            "availability_zones": [
                "nova"
            ],
            "created_at": "2018-03-19T19:17:04Z",
            "description": "",
            "distributed": False,
            "external_gateway_info": {
                "enable_snat": True,
                "external_fixed_ips": [
                    {
                        "ip_address": "172.24.4.6",
                        "subnet_id": "b930d7f6-ceb7-40a0-8b81-a425dd994ccf"
                    }
                ],
                "network_id": "ae34051f-aa6c-4c75-abf5-50dc9ac99ef3"
            },
            "flavor_id": "f7b14d9a-b0dc-4fbe-bb14-a0f4970a69e0",
            "ha": False,
            "id": "f8a44de0-fc8e-45df-93c7-f79bf3b01c95",
            "name": "router1",
            "revision_number": 3,
            "routes": [
                {
                    "destination": "179.24.1.0/24",
                    "nexthop": "172.24.3.99"
                }
            ],
            "status": "ACTIVE",
            "updated_at": "2018-03-19T19:17:22Z",
            "project_id": "0bd18306d801447bb457a46252d82d13",
            "tenant_id": "0bd18306d801447bb457a46252d82d13",
            "service_type_id": None,
            "tags": ["tag1,tag2"],
            "conntrack_helpers": []
        }
    }

    FAKE_ROUTER_ID = "f8a44de0-fc8e-45df-93c7-f79bf3b01c95"
    FAKE_INTERFACE = {
        "id": "915a14a6-867b-4af7-83d1-70efceb146f9",
        "network_id": "91c013e2-d65a-474e-9177-c3e1799ca726",
        "port_id": "2dc46bcc-d1f2-4077-b99e-91ee28afaff0",
        "subnet_id": "a2f1f29d-571b-4533-907f-5803ab96ead1",
        "subnet_ids": [
            "a2f1f29d-571b-4533-907f-5803ab96ead1"
        ],
        "project_id": "0bd18306d801447bb457a46252d82d13",
        "tenant_id": "0bd18306d801447bb457a46252d82d13",
        "tags": ["tag1,tag2"]
    }
    FAKE_INTERFACE_KWARGS = {
        "subnet_id": "a2f1f29d-571b-4533-907f-5803ab96ead1"
    }
    FAKE_SHOW_ROUTER = {
        "router": {
            "admin_state_up": "true",
            "availability_zone_hints": [],
            "availability_zones": [
                "nova"
            ],
            "created_at": "2018-03-19T19:17:04Z",
            "description": "",
            "distributed": "false",
            "external_gateway_info": {
                "enable_snat": "true",
                "external_fixed_ips": [
                    {
                        "ip_address": "172.24.4.6",
                        "subnet_id": "b930d7f6-ceb7-40a0-8b81-a425dd994ccf"
                    },
                    {
                        "ip_address": "2001:db8::9",
                        "subnet_id": "0c56df5d-ace5-46c8-8f4c-45fa4e334d18"
                    }
                ],
                "network_id": "ae34051f-aa6c-4c75-abf5-50dc9ac99ef3"
            },
            "flavor_id": "f7b14d9a-b0dc-4fbe-bb14-a0f4970a69e0",
            "ha": "false",
            "id": "f8a44de0-fc8e-45df-93c7-f79bf3b01c95",
            "name": "router1",
            "revision_number": 1,
            "routes": [
                {
                    "destination": "179.24.1.0/24",
                    "nexthop": "172.24.3.99"
                }
            ],
            "status": "ACTIVE",
            "updated_at": "2018-03-19T19:17:22Z",
            "project_id": "0bd18306d801447bb457a46252d82d13",
            "tenant_id": "0bd18306d801447bb457a46252d82d13",
            "service_type_id": "null",
            "tags": ["tag1,tag2"],
            "conntrack_helpers": []
        }
    }

    def setUp(self):
        super(TestRoutersClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = routers_client.RoutersClient(fake_auth,
                                                   'network', 'regionOne')

    def _test_list_routers(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_routers,
            'tempest.lib.common.rest_client.RestClient.get',
            {"routers": []},
            bytes_body)

    def _test_create_router(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_router,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_ROUTER,
            bytes_body,
            name="another_router", admin_state_up="true", status=201)

    def _test_show_router(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_router,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_SHOW_ROUTER,
            bytes_body,
            200,
            router_id=self.FAKE_ROUTER_ID)

    def _test_update_router(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_router,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_ROUTER,
            bytes_body,
            router_id="8604a0de-7f6b-409a-a47c-a1cc7bc77b2e",
            admin_state_up=False)

    def _test_add_router_interface(self, bytes_body=False):
        self.check_service_client_function(
            self.client.add_router_interface,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_INTERFACE,
            bytes_body,
            router_id=self.FAKE_ROUTER_ID,
            **self.FAKE_INTERFACE_KWARGS)

    def _test_remove_router_interface(self, bytes_body=False):
        self.check_service_client_function(
            self.client.remove_router_interface,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_INTERFACE,
            bytes_body,
            router_id=self.FAKE_ROUTER_ID,
            **self.FAKE_INTERFACE_KWARGS)

    def test_list_routers_with_str_body(self):
        self._test_list_routers()

    def test_list_routers_with_bytes_body(self):
        self._test_list_routers(bytes_body=True)

    def test_create_router_with_str_body(self):
        self._test_create_router()

    def test_create_router_with_bytes_body(self):
        self._test_create_router(bytes_body=True)

    def test_delete_router(self):
        self.check_service_client_function(
            self.client.delete_router,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, router_id="1", status=204)

    def test_update_router_with_str_body(self):
        self._test_update_router()

    def test_update_router_with_bytes_body(self):
        self._test_update_router(bytes_body=True)

    def test_show_router_with_str_body(self):
        self._test_show_router()

    def test_show_router_with_bytes_body(self):
        self._test_show_router(bytes_body=True)

    def test_add_router_interface_with_str_body(self):
        self._test_add_router_interface()

    def test_add_router_interface_with_bytes_body(self):
        self._test_add_router_interface(bytes_body=True)

    def test_remove_router_interface_with_str_body(self):
        self._test_remove_router_interface()

    def test_remove_router_interface_with_bytes_body(self):
        self._test_remove_router_interface(bytes_body=True)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from oslo_log import log as logging

from tempest.common import compute
from tempest.common import waiters
from tempest import config
from tempest import exceptions
from tempest.lib.common import api_version_request
from tempest.lib.common import api_version_utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions as lib_exc
import tempest.test

CONF = config.CONF

LOG = logging.getLogger(__name__)


class BaseV2ComputeTest(api_version_utils.BaseMicroversionTest,
                        tempest.test.BaseTestCase):
    """Base test case class for all Compute API tests."""

    force_tenant_isolation = False
    # Set this to True in subclasses to create a default network. See
    # https://bugs.launchpad.net/tempest/+bug/1844568
    create_default_network = False

    # TODO(andreaf) We should care also for the alt_manager here
    # but only once client lazy load in the manager is done
    credentials = ['primary']

    @classmethod
    def skip_checks(cls):
        super(BaseV2ComputeTest, cls).skip_checks()
        if not CONF.service_available.nova:
            raise cls.skipException("Nova is not available")
        if cls.create_default_network and not CONF.service_available.neutron:
            raise cls.skipException("Neutron is not available")

        api_version_utils.check_skip_with_microversion(
            cls.min_microversion, cls.max_microversion,
            CONF.compute.min_microversion, CONF.compute.max_microversion)
        api_version_utils.check_skip_with_microversion(
            cls.volume_min_microversion, cls.volume_max_microversion,
            CONF.volume.min_microversion, CONF.volume.max_microversion)
        api_version_utils.check_skip_with_microversion(
            cls.placement_min_microversion, cls.placement_max_microversion,
            CONF.placement.min_microversion, CONF.placement.max_microversion)

    @classmethod
    def setup_credentials(cls):
        # Setting network=True, subnet=True creates a default network
        cls.set_network_resources(
            network=cls.create_default_network,
            subnet=cls.create_default_network,
            router=cls.create_default_network,
            dhcp=cls.create_default_network)
        super(BaseV2ComputeTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(BaseV2ComputeTest, cls).setup_clients()
        cls.servers_client = cls.os_primary.servers_client
        cls.server_groups_client = cls.os_primary.server_groups_client
        cls.flavors_client = cls.os_primary.flavors_client
        cls.compute_images_client = cls.os_primary.compute_images_client
        cls.extensions_client = cls.os_primary.extensions_client
        cls.floating_ips_client = cls.os_primary.compute_floating_ips_client
        cls.keypairs_client = cls.os_primary.keypairs_client
        cls.security_group_rules_client = (
            cls.os_primary.compute_security_group_rules_client)
        cls.security_groups_client =\
            cls.os_primary.compute_security_groups_client
        cls.quotas_client = cls.os_primary.quotas_client
        cls.compute_networks_client = cls.os_primary.compute_networks_client
        cls.limits_client = cls.os_primary.limits_client
        cls.volumes_extensions_client =\
            cls.os_primary.volumes_extensions_client
        cls.snapshots_extensions_client =\
            cls.os_primary.snapshots_extensions_client
        cls.interfaces_client = cls.os_primary.interfaces_client
        cls.availability_zone_client = cls.os_primary.availability_zone_client
        cls.agents_client = cls.os_primary.agents_client
        cls.aggregates_client = cls.os_primary.aggregates_client
        cls.services_client = cls.os_primary.services_client
        cls.instance_usages_audit_log_client = (
            cls.os_primary.instance_usages_audit_log_client)
        cls.hypervisor_client = cls.os_primary.hypervisor_client
        cls.certificates_client = cls.os_primary.certificates_client
        cls.migrations_client = cls.os_primary.migrations_client
        cls.security_group_default_rules_client = (
            cls.os_primary.security_group_default_rules_client)
        cls.versions_client = cls.os_primary.compute_versions_client
        if CONF.service_available.cinder:
            cls.volumes_client = cls.os_primary.volumes_client_latest
            cls.attachments_client = cls.os_primary.attachments_client_latest
            cls.snapshots_client = cls.os_primary.snapshots_client_latest
        if CONF.service_available.glance:
            if CONF.image_feature_enabled.api_v2:
                cls.images_client = cls.os_primary.image_client_v2
            else:
                raise lib_exc.InvalidConfiguration(
                    'api_v2 must be True in [image-feature-enabled].')

    @classmethod
    def resource_setup(cls):
        super(BaseV2ComputeTest, cls).resource_setup()
        cls.request_microversion = (
            api_version_utils.select_request_microversion(
                cls.min_microversion,
                CONF.compute.min_microversion))
        cls.volume_request_microversion = (
            api_version_utils.select_request_microversion(
                cls.volume_min_microversion,
                CONF.volume.min_microversion))
        cls.placement_request_microversion = (
            api_version_utils.select_request_microversion(
                cls.placement_min_microversion,
                CONF.placement.min_microversion))
        cls.setup_api_microversion_fixture(
            compute_microversion=cls.request_microversion,
            volume_microversion=cls.volume_request_microversion,
            placement_microversion=cls.placement_request_microversion)

        cls.build_interval = CONF.compute.build_interval
        cls.build_timeout = CONF.compute.build_timeout
        cls.image_ref = CONF.compute.image_ref
        cls.image_ref_alt = CONF.compute.image_ref_alt
        cls.flavor_ref = CONF.compute.flavor_ref
        cls.flavor_ref_alt = CONF.compute.flavor_ref_alt
        cls.ssh_user = CONF.validation.image_ssh_user
        cls.ssh_alt_user = CONF.validation.image_alt_ssh_user
        cls.image_ssh_user = CONF.validation.image_ssh_user
        cls.image_alt_ssh_user = CONF.validation.image_alt_ssh_user
        cls.image_ssh_password = CONF.validation.image_ssh_password
        cls.image_alt_ssh_password = CONF.validation.image_alt_ssh_password

    @classmethod
    def is_requested_microversion_compatible(cls, max_version):
        """Check the compatibility of selected request microversion

        This method will check if selected request microversion
        (cls.request_microversion) for test is compatible with respect
        to 'max_version'. Compatible means if selected request microversion
        is in the range(<=) of 'max_version'.

        :param max_version: maximum microversion to compare for compatibility.
            Example: '2.30'
        :returns: True if selected request microversion is compatible with
            'max_version'. False in other case.
        """
        try:
            req_version_obj = api_version_request.APIVersionRequest(
                cls.request_microversion)
        # NOTE(gmann): This is case where this method is used before calling
        # resource_setup(), where cls.request_microversion is set. There may
        # not be any such case but still we can handle this case.
        except AttributeError:
            request_microversion = (
                api_version_utils.select_request_microversion(
                    cls.min_microversion,
                    CONF.compute.min_microversion))
            req_version_obj = api_version_request.APIVersionRequest(
                request_microversion)
        max_version_obj = api_version_request.APIVersionRequest(max_version)
        return req_version_obj <= max_version_obj

    @classmethod
    def server_check_teardown(cls):
        """Checks is the shared server clean enough for subsequent test.

           Method will delete the server when it's dirty.
           The setUp method is responsible for creating a new server.
           Exceptions raised in tearDown class are fails the test case,
           This method supposed to use only by tearDown methods, when
           the shared server_id is stored in the server_id of the class.
        """
        if getattr(cls, 'server_id', None) is not None:
            try:
                waiters.wait_for_server_status(cls.servers_client,
                                               cls.server_id, 'ACTIVE')
            except Exception as exc:
                LOG.exception(exc)
                cls.servers_client.delete_server(cls.server_id)
                waiters.wait_for_server_termination(cls.servers_client,
                                                    cls.server_id)
                cls.server_id = None
                raise

    @classmethod
    def create_test_server(cls, validatable=False, volume_backed=False,
                           validation_resources=None, clients=None, **kwargs):
        """Wrapper utility that returns a test server.

        This wrapper utility calls the common create test server and
        returns a test server. The purpose of this wrapper is to minimize
        the impact on the code of the tests already using this
        function.

        :param validatable: Whether the server will be pingable or sshable.
        :param volume_backed: Whether the instance is volume backed or not.
        :param validation_resources: Dictionary of validation resources as
            returned by `get_class_validation_resources`.
        :param clients: Client manager, defaults to os_primary.
        :param kwargs: Extra arguments are passed down to the
            `compute.create_test_server` call.
        """
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=cls.__name__ + "-server")

        request_version = api_version_request.APIVersionRequest(
            cls.request_microversion)
        v2_37_version = api_version_request.APIVersionRequest('2.37')

        tenant_network = cls.get_tenant_network()
        # NOTE(snikitin): since microversion v2.37 'networks' field is required
        if (request_version >= v2_37_version and 'networks' not in kwargs and
            not tenant_network):
            kwargs['networks'] = 'none'

        if clients is None:
            clients = cls.os_primary

        body, servers = compute.create_test_server(
            clients,
            validatable,
            validation_resources=validation_resources,
            tenant_network=tenant_network,
            volume_backed=volume_backed,
            **kwargs)

        # For each server schedule wait and delete, so we first delete all
        # and then wait for all
        for server in servers:
            cls.addClassResourceCleanup(waiters.wait_for_server_termination,
                                        clients.servers_client, server['id'])
        for server in servers:
            cls.addClassResourceCleanup(
                test_utils.call_and_ignore_notfound_exc,
                clients.servers_client.delete_server, server['id'])

        return body

    @classmethod
    def create_security_group(cls, name=None, description=None):
        prefix = CONF.resource_name_prefix
        if name is None:
            name = data_utils.rand_name(
                prefix=prefix, name=cls.__name__ + "-securitygroup")
        if description is None:
            description = data_utils.rand_name(
                prefix=prefix, name='description')
        body = cls.security_groups_client.create_security_group(
            name=name, description=description)['security_group']
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.security_groups_client.delete_security_group,
            body['id'])

        return body

    @classmethod
    def create_test_server_group(cls, name="", policy=None):
        if not name:
            name = data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=cls.__name__ + "-Server-Group")
        if cls.is_requested_microversion_compatible('2.63'):
            policy = policy or ['affinity']
            if not isinstance(policy, list):
                policy = [policy]
            kwargs = {'policies': policy}
        else:
            policy = policy or 'affinity'
            if isinstance(policy, list):
                policy = policy[0]
            kwargs = {'policy': policy}
        body = cls.server_groups_client.create_server_group(
            name=name, **kwargs)['server_group']
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.server_groups_client.delete_server_group,
            body['id'])
        return body

    def wait_for(self, condition, *args):
        """Repeatedly calls condition() until a timeout."""
        start_time = int(time.time())
        while True:
            try:
                condition(*args)
            except Exception:
                pass
            else:
                return
            if int(time.time()) - start_time >= self.build_timeout:
                condition(*args)
                return
            time.sleep(self.build_interval)

    @classmethod
    def prepare_instance_network(cls):
        if (CONF.validation.auth_method != 'disabled' and
                CONF.validation.connect_method == 'floating'):
            cls.set_network_resources(network=True, subnet=True, router=True,
                                      dhcp=True)

    @classmethod
    def create_image_from_server(cls, server_id, **kwargs):
        """Wrapper utility that returns an image created from the server.

        If compute microversion >= 2.36, the returned image response will
        be from the image service API rather than the compute image proxy API.
        """
        name = kwargs.pop(
            'name',
            data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=cls.__name__ + "-image"))
        wait_until = kwargs.pop('wait_until', None)
        wait_for_server = kwargs.pop('wait_for_server', True)

        image = cls.compute_images_client.create_image(server_id, name=name,
                                                       **kwargs)
        if api_version_utils.compare_version_header_to_response(
            "OpenStack-API-Version", "compute 2.45", image.response, "lt"):
            image_id = image['image_id']
        else:
            image_id = data_utils.parse_image_id(image.response['location'])

        # The compute image proxy APIs were deprecated in 2.35 so
        # use the images client directly if the API microversion being
        # used is >=2.36.
        if not cls.is_requested_microversion_compatible('2.35'):
            client = cls.images_client
        else:
            client = cls.compute_images_client
        cls.addClassResourceCleanup(test_utils.call_and_ignore_notfound_exc,
                                    client.delete_image, image_id)

        if wait_until is not None:
            try:
                wait_until = wait_until.upper()
                if not cls.is_requested_microversion_compatible('2.35'):
                    wait_until = wait_until.lower()
                waiters.wait_for_image_status(client, image_id, wait_until)
            except lib_exc.NotFound:
                if wait_until.upper() == 'ACTIVE':
                    # If the image is not found after create_image returned
                    # that means the snapshot failed in nova-compute and nova
                    # deleted the image. There should be a compute fault
                    # recorded with the server in that case, so get the server
                    # and dump some details.
                    server = (
                        cls.servers_client.show_server(server_id)['server'])
                    if 'fault' in server:
                        raise exceptions.SnapshotNotFoundException(
                            server['fault'], image_id=image_id)
                    else:
                        raise exceptions.SnapshotNotFoundException(
                            image_id=image_id)
                else:
                    raise
            image = client.show_image(image_id)
            # Compute image client returns response wrapped in 'image' element
            # which is not the case with Glance image client.
            if 'image' in image:
                image = image['image']

            if wait_until.upper() == 'ACTIVE':
                if wait_for_server:
                    waiters.wait_for_server_status(cls.servers_client,
                                                   server_id, 'ACTIVE')
        return image

    @classmethod
    def recreate_server(cls, server_id, validatable=False, wait_until='ACTIVE',
                        **kwargs):
        """Destroy an existing class level server and creates a new one

        Some test classes use a test server that can be used by multiple
        tests. This is done to optimise runtime and test load.
        If something goes wrong with the test server, it can be rebuilt
        using this helper.

        This helper can also be used for the initial provisioning if no
        server_id is specified.

        :param server_id: UUID of the server to be rebuilt. If None is
            specified, a new server is provisioned.
        :param validatable: whether to the server needs to be
            validatable. When True, validation resources are acquired via
            the `get_class_validation_resources` helper.
        :param kwargs: extra parameters are passed through to the
            `create_test_server` call.
        :return: the UUID of the created server.
        """
        if server_id:
            cls.delete_server(server_id)

        cls.password = data_utils.rand_password()
        server = cls.create_test_server(
            validatable,
            validation_resources=cls.get_class_validation_resources(
                cls.os_primary),
            wait_until=wait_until,
            adminPass=cls.password,
            **kwargs)
        return server['id']

    @classmethod
    def delete_server(cls, server_id):
        """Deletes an existing server and waits for it to be gone."""
        try:
            cls.servers_client.delete_server(server_id)
            waiters.wait_for_server_termination(cls.servers_client,
                                                server_id)
        except Exception:
            LOG.exception('Failed to delete server %s', server_id)

    def resize_server(
        self, server_id, new_flavor_id, wait_until='ACTIVE', **kwargs
    ):
        """resize and confirm_resize an server, waits for it to be ACTIVE."""
        body = self.servers_client.resize_server(
            server_id, new_flavor_id, **kwargs)
        waiters.wait_for_server_status(
            self.servers_client, server_id, 'VERIFY_RESIZE',
            request_id=body.response['x-openstack-request-id'])
        self.servers_client.confirm_resize_server(server_id)

        waiters.wait_for_server_status(
            self.servers_client, server_id, 'ACTIVE')
        server = self.servers_client.show_server(server_id)['server']

        validation_resources = self.get_class_validation_resources(
            self.os_primary)
        if (
            validation_resources and
            wait_until in ("SSHABLE", "PINGABLE") and
            CONF.validation.run_validation
        ):
            tenant_network = self.get_tenant_network()
            compute.wait_for_ssh_or_ping(
                server, self.os_primary, tenant_network,
                True, validation_resources, wait_until, True)

        self.assert_flavor_equal(new_flavor_id, server['flavor'])

    def reboot_server(self, server_id, type):
        """Reboot a server and wait for it to be ACTIVE."""
        self.servers_client.reboot_server(server_id, type=type)
        waiters.wait_for_server_status(
            self.servers_client, server_id, 'ACTIVE')

    @classmethod
    def delete_volume(cls, volume_id):
        """Deletes the given volume and waits for it to be gone."""
        try:
            cls.volumes_client.delete_volume(volume_id)
            # TODO(mriedem): We should move the wait_for_resource_deletion
            # into the delete_volume method as a convenience to the caller.
            cls.volumes_client.wait_for_resource_deletion(volume_id)
        except lib_exc.NotFound:
            LOG.warning("Unable to delete volume '%s' since it was not found. "
                        "Maybe it was already deleted?", volume_id)

    @classmethod
    def get_server_ip(cls, server, validation_resources=None):
        """Get the server fixed or floating IP.

        Based on the configuration we're in, return a correct ip
        address for validating that a guest is up.

        :param server: The server dict as returned by the API
        :param validation_resources: The dict of validation resources
            provisioned for the server.
        """
        return compute.get_server_ip(
            server, validation_resources=validation_resources)

    @classmethod
    def create_volume(cls, image_ref=None, **kwargs):
        """Create a volume and wait for it to become 'available'.

        :param image_ref: Specify an image id to create a bootable volume.
        :param wait_for_available: Wait until the volume becomes available
               before returning
        :param kwargs: other parameters to create volume.
        :returns: The available volume.
        """
        if 'size' not in kwargs:
            kwargs['size'] = CONF.volume.volume_size
        if 'display_name' not in kwargs:
            vol_name = data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=cls.__name__ + '-volume')
            kwargs['display_name'] = vol_name
        if image_ref is not None:
            kwargs['imageRef'] = image_ref
        wait = kwargs.pop('wait_for_available', True)
        if CONF.volume.volume_type and 'volume_type' not in kwargs:
            # If volume_type is not provided in config then no need to
            # add a volume type and
            # if volume_type has already been added by child class then
            # no need to override.
            kwargs['volume_type'] = CONF.volume.volume_type
        if CONF.compute.compute_volume_common_az:
            kwargs.setdefault('availability_zone',
                              CONF.compute.compute_volume_common_az)
        volume = cls.volumes_client.create_volume(**kwargs)['volume']
        cls.addClassResourceCleanup(
            cls.volumes_client.wait_for_resource_deletion, volume['id'])
        cls.addClassResourceCleanup(test_utils.call_and_ignore_notfound_exc,
                                    cls.volumes_client.delete_volume,
                                    volume['id'])
        if wait:
            waiters.wait_for_volume_resource_status(cls.volumes_client,
                                                    volume['id'], 'available')
        return volume

    @classmethod
    def verify_metadata_from_api(self, server, ssh_client, verify_method):
        md_url = 'http://169.254.169.254/openstack/latest/meta_data.json'
        LOG.info('Attempting to verify tagged devices in server %s via '
                 'the metadata service: %s', server['id'], md_url)

        def get_and_verify_metadata():
            try:
                ssh_client.exec_command('curl -V')
            except lib_exc.SSHExecCommandFailed:
                if not CONF.compute_feature_enabled.config_drive:
                    raise self.skipException('curl not found in guest '
                                             'and config drive is '
                                             'disabled')
                LOG.warning('curl was not found in the guest, device '
                            'tagging metadata was not checked in the '
                            'metadata API')
                return True
            cmd = 'curl %s' % md_url
            md_json = ssh_client.exec_command(cmd)
            return verify_method(md_json)
        # NOTE(gmann) Keep refreshing the metadata info until the metadata
        # cache is refreshed. For safer side, we will go with wait loop of
        # build_interval till build_timeout. verify_method() above will return
        # True if all metadata verification is done as expected.
        if not test_utils.call_until_true(get_and_verify_metadata,
                                          CONF.compute.build_timeout,
                                          CONF.compute.build_interval):
            raise lib_exc.TimeoutException('Timeout while verifying '
                                           'metadata on server.')

    def _detach_volume(self, server, volume):
        """Helper method to detach a volume.

        Ignores 404 responses if the volume or server do not exist, or the
        volume is already detached from the server.
        """
        try:
            volume = self.volumes_client.show_volume(volume['id'])['volume']
            # Check the status. You can only detach an in-use volume, otherwise
            # the compute API will return a 400 response.
            if volume['status'] == 'in-use':
                self.servers_client.detach_volume(server['id'], volume['id'])
        except lib_exc.NotFound:
            # Ignore 404s on detach in case the server is deleted or the volume
            # is already detached.
            pass

    def attach_volume(self, server, volume, device=None, tag=None,
                      wait_for_detach=True):
        """Attaches volume to server and waits for 'in-use' volume status.

        The volume will be detached when the test tears down.

        :param server: The server to which the volume will be attached.
        :param volume: The volume to attach.
        :param device: Optional mountpoint for the attached volume. Note that
            this is not guaranteed for all hypervisors and is not recommended.
        :param tag: Optional device role tag to apply to the volume.
        """
        attach_kwargs = dict(volumeId=volume['id'])
        if device:
            attach_kwargs['device'] = device
        if tag:
            attach_kwargs['tag'] = tag

        attachment = self.servers_client.attach_volume(
            server['id'], **attach_kwargs)['volumeAttachment']

        # NOTE(lyarwood): During attach we initially wait for the volume
        # attachment and then check the volume state.
        waiters.wait_for_volume_attachment_create(
            self.volumes_client, volume['id'], server['id'])
        # TODO(lyarwood): Remove the following volume status checks and move to
        # attachment status checks across all volumes now with the 3.27
        # microversion somehow.
        if not volume['multiattach']:
            waiters.wait_for_volume_resource_status(
                self.volumes_client, volume['id'], 'in-use')

        # NOTE(lyarwood): On teardown (LIFO) initially wait for the volume
        # attachment in Nova to be removed. While this technically happens last
        # we want this to be the first waiter as if it fails we can then dump
        # the contents of the console log. The final check of the volume state
        # should be a no-op by this point and is just added for completeness
        # when detaching non-multiattach volumes.
        if not volume['multiattach'] and wait_for_detach:
            self.addCleanup(
                waiters.wait_for_volume_resource_status, self.volumes_client,
                volume['id'], 'available')
        self.addCleanup(
            waiters.wait_for_volume_attachment_remove_from_server,
            self.servers_client, server['id'], volume['id'])
        self.addCleanup(self._detach_volume, server, volume)

        return attachment

    def create_volume_snapshot(self, volume_id, name=None, description=None,
                               metadata=None, force=False):
        name = name or data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-snapshot')
        snapshot = self.snapshots_client.create_snapshot(
            volume_id=volume_id,
            force=force,
            display_name=name,
            description=description,
            metadata=metadata)['snapshot']
        self.addCleanup(self.snapshots_client.wait_for_resource_deletion,
                        snapshot['id'])
        self.addCleanup(self.snapshots_client.delete_snapshot, snapshot['id'])
        waiters.wait_for_volume_resource_status(self.snapshots_client,
                                                snapshot['id'], 'available')
        snapshot = self.snapshots_client.show_snapshot(
            snapshot['id'])['snapshot']
        return snapshot

    def assert_flavor_equal(self, flavor_id, server_flavor):
        """Check whether server_flavor equals to flavor.

        :param flavor_id: flavor id
        :param server_flavor: flavor info returned by show_server.
        """
        # Nova API > 2.46 no longer includes flavor.id, and schema check
        # will cover whether 'id' should be in flavor
        if server_flavor.get('id'):
            msg = ('server flavor is not same as flavor!')
            self.assertEqual(flavor_id, server_flavor['id'], msg)
        else:
            flavor = self.flavors_client.show_flavor(flavor_id)['flavor']
            self.assertEqual(flavor['name'], server_flavor['original_name'],
                             "original_name in server flavor is not same as "
                             "flavor name!")
            for key in ['ram', 'vcpus', 'disk']:
                msg = ('attribute %s in server flavor is not same as '
                       'flavor!' % key)
                self.assertEqual(flavor[key], server_flavor[key], msg)


class BaseV2ComputeAdminTest(BaseV2ComputeTest):
    """Base test case class for Compute Admin API tests."""

    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(BaseV2ComputeAdminTest, cls).setup_clients()
        cls.availability_zone_admin_client = (
            cls.os_admin.availability_zone_client)
        cls.admin_flavors_client = cls.os_admin.flavors_client
        cls.admin_servers_client = cls.os_admin.servers_client
        cls.admin_image_client = cls.os_admin.image_client_v2
        cls.admin_assisted_volume_snapshots_client = \
            cls.os_admin.assisted_volume_snapshots_client

    def create_flavor(self, ram, vcpus, disk, name=None,
                      is_public='True', **kwargs):
        if name is None:
            name = data_utils.rand_name(
                prefix=CONF.resource_name_prefix,
                name=self.__class__.__name__ + "-flavor")
        id = kwargs.pop('id', data_utils.rand_int_id(start=1000))
        client = self.admin_flavors_client
        flavor = client.create_flavor(
            ram=ram, vcpus=vcpus, disk=disk, name=name,
            id=id, is_public=is_public, **kwargs)['flavor']
        self.addCleanup(client.wait_for_resource_deletion, flavor['id'])
        self.addCleanup(client.delete_flavor, flavor['id'])
        return flavor

    @classmethod
    def get_host_for_server(cls, server_id):
        server_details = cls.admin_servers_client.show_server(server_id)
        return server_details['server']['OS-EXT-SRV-ATTR:host']

    def get_host_other_than(self, server_id):
        source_host = self.get_host_for_server(server_id)

        svcs = self.os_admin.services_client.list_services(
            binary='nova-compute')['services']
        hosts = []
        for svc in svcs:
            if CONF.compute.target_hosts_to_avoid in svc['host']:
                continue
            if svc['state'] == 'up' and svc['status'] == 'enabled':
                if CONF.compute.compute_volume_common_az:
                    if svc['zone'] == CONF.compute.compute_volume_common_az:
                        hosts.append(svc['host'])
                else:
                    hosts.append(svc['host'])

        for target_host in hosts:
            if source_host != target_host:
                return target_host



# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class ServersAdminTestJSON(base.BaseV2ComputeAdminTest):
    """Tests Servers API using admin privileges"""

    create_default_network = True

    @classmethod
    def setup_clients(cls):
        super(ServersAdminTestJSON, cls).setup_clients()
        cls.client = cls.os_admin.servers_client
        cls.non_admin_client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(ServersAdminTestJSON, cls).resource_setup()

        prefix = CONF.resource_name_prefix
        cls.s1_name = data_utils.rand_name(prefix=prefix,
                                           name=cls.__name__ + '-server')
        server = cls.create_test_server(name=cls.s1_name)
        cls.s1_id = server['id']

        cls.s2_name = data_utils.rand_name(prefix=prefix,
                                           name=cls.__name__ + '-server')
        server = cls.create_test_server(name=cls.s2_name,
                                        wait_until='ACTIVE')
        cls.s2_id = server['id']
        waiters.wait_for_server_status(cls.non_admin_client,
                                       cls.s1_id, 'ACTIVE')

    @decorators.idempotent_id('06f960bb-15bb-48dc-873d-f96e89be7870')
    def test_list_servers_filter_by_error_status(self):
        """Test filtering the list of servers by server error status"""
        params = {'status': 'error'}
        self.client.reset_state(self.s1_id, state='error')
        body = self.non_admin_client.list_servers(**params)
        # Reset server's state to 'active'
        self.client.reset_state(self.s1_id, state='active')
        # Verify server's state
        server = self.client.show_server(self.s1_id)['server']
        self.assertEqual(server['status'], 'ACTIVE')
        servers = body['servers']
        # Verify error server in list result
        self.assertIn(self.s1_id, map(lambda x: x['id'], servers))
        self.assertNotIn(self.s2_id, map(lambda x: x['id'], servers))

    @decorators.idempotent_id('d56e9540-73ed-45e0-9b88-98fc419087eb')
    def test_list_servers_detailed_filter_by_invalid_status(self):
        """Test filtering the list of servers by invalid server status"""
        params = {'status': 'invalid_status'}
        if self.is_requested_microversion_compatible('2.37'):
            body = self.client.list_servers(detail=True, **params)
            servers = body['servers']
            self.assertEmpty(servers)
        else:
            self.assertRaises(lib_exc.BadRequest, self.client.list_servers,
                              detail=True, **params)

    @decorators.idempotent_id('51717b38-bdc1-458b-b636-1cf82d99f62f')
    def test_list_servers_by_admin(self):
        """Test listing servers by admin without other projects

        Listing servers by admin user returns a list which doesn't
        contain the other projects' server by default.
        """
        body = self.client.list_servers(detail=True)
        servers = body['servers']

        # This case is for the test environments which contain
        # the existing servers before testing
        servers_name = [server['name'] for server in servers]
        self.assertNotIn(self.s1_name, servers_name)
        self.assertNotIn(self.s2_name, servers_name)

    @decorators.idempotent_id('9f5579ae-19b4-4985-a091-2a5d56106580')
    def test_list_servers_by_admin_with_all_tenants(self):
        """Test listing servers by admin with all tenants

        Listing servers by admin user with all tenants parameter,
        all servers should be listed.
        """
        params = {'all_tenants': ''}
        body = self.client.list_servers(detail=True, **params)
        servers = body['servers']
        servers_name = [server['name'] for server in servers]

        self.assertIn(self.s1_name, servers_name)
        self.assertIn(self.s2_name, servers_name)

    @decorators.related_bug('1659811')
    @decorators.idempotent_id('7e5d6b8f-454a-4ba1-8ae2-da857af8338b')
    def test_list_servers_by_admin_with_specified_tenant(self):
        """Test listing servers by admin with specified project

        In nova v2, tenant_id is ignored unless all_tenants is specified.
        """
        # List the primary tenant but get nothing due to odd specified behavior
        tenant_id = self.non_admin_client.tenant_id
        params = {'tenant_id': tenant_id}
        body = self.client.list_servers(detail=True, **params)
        servers = body['servers']
        servers_name = [x['name'] for x in servers]
        self.assertNotIn(self.s1_name, servers_name)
        self.assertNotIn(self.s2_name, servers_name)

        # List the primary tenant with all_tenants is specified
        params = {'all_tenants': '', 'tenant_id': tenant_id}
        body = self.client.list_servers(detail=True, **params)
        servers = body['servers']
        servers_name = [x['name'] for x in servers]
        self.assertIn(self.s1_name, servers_name)
        self.assertIn(self.s2_name, servers_name)

        # List the admin tenant shouldn't get servers created by other tenants
        admin_tenant_id = self.client.tenant_id
        params = {'all_tenants': '', 'tenant_id': admin_tenant_id}
        body = self.client.list_servers(detail=True, **params)
        servers = body['servers']
        servers_name = [x['name'] for x in servers]
        self.assertNotIn(self.s1_name, servers_name)
        self.assertNotIn(self.s2_name, servers_name)

    @decorators.idempotent_id('86c7a8f7-50cf-43a9-9bac-5b985317134f')
    def test_list_servers_filter_by_exist_host(self):
        """Test filtering the list of servers by existent host"""
        server = self.client.show_server(self.s1_id)['server']
        hostname = server['OS-EXT-SRV-ATTR:host']
        params = {'host': hostname, 'all_tenants': '1'}
        servers = self.client.list_servers(**params)['servers']
        self.assertIn(server['id'], map(lambda x: x['id'], servers))

        nonexistent_params = {'host': 'nonexistent_host',
                              'all_tenants': '1'}
        nonexistent_body = self.client.list_servers(**nonexistent_params)
        nonexistent_servers = nonexistent_body['servers']
        self.assertNotIn(server['id'],
                         map(lambda x: x['id'], nonexistent_servers))

    @decorators.idempotent_id('ee8ae470-db70-474d-b752-690b7892cab1')
    def test_reset_state_server(self):
        """Test resetting server state to error/active"""
        # Reset server's state to 'error'
        self.client.reset_state(self.s1_id, state='error')

        # Verify server's state
        server = self.client.show_server(self.s1_id)['server']
        self.assertEqual(server['status'], 'ERROR')

        # Reset server's state to 'active'
        self.client.reset_state(self.s1_id, state='active')

        # Verify server's state
        server = self.client.show_server(self.s1_id)['server']
        self.assertEqual(server['status'], 'ACTIVE')

    @decorators.idempotent_id('682cb127-e5bb-4f53-87ce-cb9003604442')
    def test_rebuild_server_in_error_state(self):
        """Test rebuilding server in error state

        The server in error state should be rebuilt using the provided
        image and changed to ACTIVE state.
        """
        # resetting vm state require admin privilege
        self.client.reset_state(self.s1_id, state='error')
        rebuilt_server = self.non_admin_client.rebuild_server(
            self.s1_id, self.image_ref_alt)['server']
        self.addCleanup(waiters.wait_for_server_status, self.non_admin_client,
                        self.s1_id, 'ACTIVE')
        self.addCleanup(self.non_admin_client.rebuild_server, self.s1_id,
                        self.image_ref)

        # Verify the properties in the initial response are correct
        self.assertEqual(self.s1_id, rebuilt_server['id'])
        rebuilt_image_id = rebuilt_server['image']['id']
        self.assertEqual(self.image_ref_alt, rebuilt_image_id)
        self.assert_flavor_equal(self.flavor_ref, rebuilt_server['flavor'])
        waiters.wait_for_server_status(self.non_admin_client,
                                       rebuilt_server['id'], 'ACTIVE',
                                       raise_on_error=False)
        # Verify the server properties after rebuilding
        server = (self.non_admin_client.show_server(rebuilt_server['id'])
                  ['server'])
        rebuilt_image_id = server['image']['id']
        self.assertEqual(self.image_ref_alt, rebuilt_image_id)

    @decorators.idempotent_id('7a1323b4-a6a2-497a-96cb-76c07b945c71')
    def test_inject_network_info(self):
        """Test injecting network info of a server"""
        # Create a server
        server = self.create_test_server(wait_until='ACTIVE')
        # Inject the Network Info into Server
        self.client.inject_network_info(server['id'])

    @decorators.idempotent_id('fdcd9b33-0903-4e00-a1f7-b5f6543068d6')
    def test_create_server_with_scheduling_hint(self):
        """Test creating server with scheduling hint"""
        # Create a server with scheduler hints.
        hints = {
            'same_host': self.s1_id
        }
        self.create_test_server(scheduler_hints=hints,
                                wait_until='ACTIVE')


class ServersAdmin275Test(base.BaseV2ComputeAdminTest):
    """Test compute server with microversion greater than 2.75

    # NOTE(gmann): This test tests the Server APIs response schema
    # for 2.75 microversion. No specific assert or behaviour verification
    # is needed.
    """

    min_microversion = '2.75'

    @decorators.idempotent_id('bf2b4a00-73a3-4d53-81fa-acbcd97d6339')
    def test_rebuild_update_server_275(self):
        server = self.create_test_server()
        # Checking update response schema.
        self.servers_client.update_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'ACTIVE')
        # Checking rebuild API response schema
        self.servers_client.rebuild_server(server['id'], self.image_ref_alt)
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')
        # Checking rebuild server with admin response schema.
        self.os_admin.servers_client.rebuild_server(
            server['id'], self.image_ref)
        self.addCleanup(waiters.wait_for_server_status,
                        self.os_admin.servers_client,
                        server['id'], 'ACTIVE')



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import operator

from tempest.api.volume import base
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class VolumeTypesAccessTest(base.BaseVolumeAdminTest):
    """Test volume type access"""

    credentials = ['primary', 'alt', 'admin']

    @decorators.idempotent_id('d4dd0027-835f-4554-a6e5-50903fb79184')
    def test_volume_type_access_add(self):
        """Test adding volume type access for non-admin project"""
        # Creating a NON public volume type
        params = {'os-volume-type-access:is_public': False}
        volume_type = self.create_volume_type(**params)

        # Try creating a volume from volume type in primary tenant
        self.assertRaises(lib_exc.NotFound, self.volumes_client.create_volume,
                          volume_type=volume_type['id'],
                          size=CONF.volume.volume_size)

        # Adding volume type access for primary tenant
        self.admin_volume_types_client.add_type_access(
            volume_type['id'], project=self.volumes_client.tenant_id)
        self.addCleanup(self.admin_volume_types_client.remove_type_access,
                        volume_type['id'],
                        project=self.volumes_client.tenant_id)

        # Creating a volume from primary tenant
        volume = self.create_volume(volume_type=volume_type['id'])
        # Validating the created volume is based on the volume type
        self.assertEqual(volume_type['name'], volume['volume_type'])

    @decorators.idempotent_id('5220eb28-a435-43ce-baaf-ed46f0e95159')
    def test_volume_type_access_list(self):
        """Test listing volume type access"""
        # Creating a NON public volume type
        params = {'os-volume-type-access:is_public': False}
        volume_type = self.create_volume_type(**params)

        # Adding volume type access for primary tenant
        self.admin_volume_types_client.add_type_access(
            volume_type['id'], project=self.volumes_client.tenant_id)
        self.addCleanup(self.admin_volume_types_client.remove_type_access,
                        volume_type['id'],
                        project=self.volumes_client.tenant_id)

        # Adding volume type access for alt tenant
        self.admin_volume_types_client.add_type_access(
            volume_type['id'],
            project=self.os_alt.volumes_client_latest.tenant_id)
        self.addCleanup(self.admin_volume_types_client.remove_type_access,
                        volume_type['id'],
                        project=self.os_alt.volumes_client_latest.tenant_id)

        # List tenant access for the given volume type
        type_access_list = self.admin_volume_types_client.list_type_access(
            volume_type['id'])['volume_type_access']
        volume_type_ids = [
            vol_type['volume_type_id'] for vol_type in type_access_list
        ]

        # Validating volume type available for only two tenants
        self.assertEqual(2, volume_type_ids.count(volume_type['id']))

        # Validating the permitted tenants are the expected tenants
        self.assertIn(self.volumes_client.tenant_id,
                      map(operator.itemgetter('project_id'), type_access_list))
        self.assertIn(self.os_alt.volumes_client_latest.tenant_id,
                      map(operator.itemgetter('project_id'), type_access_list))



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.volume import volumes


manage_volume = {
    'status_code': [202],
    'response_body': {
        'type': 'object',
        'properties': {
            'volume': volumes.common_show_volume},
        'additionalProperties': False,
        'required': ['volume']
    }
}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_96 import servers as servers296


###########################################################################
#
# 2.98:
#
# The image properties parameter is now returned in the response body of the
# following calls:
#
# - GET /servers/detail
# - GET /servers/{server_id}
# - PUT /servers/{server_id}
# - POST /servers/{server_id}/action (rebuild)
#
###########################################################################

image_properties = {
    'type': 'object',
    'patternProperties': {
        '^[a-zA-Z0-9_:. ]{1,255}$': {
            'oneOf': [
                {'type': 'string', 'maxLength': 255},
                {'type': 'null'},
            ]
        },
    },
    'additionalProperties': False,
}

get_server = copy.deepcopy(servers296.get_server)
get_server['response_body']['properties']['server']['properties'][
    'image']['oneOf'][0]['properties'].update({'properties': image_properties})

list_servers_detail = copy.deepcopy(servers296.list_servers_detail)
list_servers_detail['response_body']['properties']['servers']['items'][
    'properties']['image']['oneOf'][0]['properties'].update(
        {'properties': image_properties})

update_server = copy.deepcopy(servers296.update_server)
update_server['response_body']['properties']['server']['properties'][
    'image']['oneOf'][0]['properties'].update({'properties': image_properties})

rebuild_server = copy.deepcopy(servers296.rebuild_server)
rebuild_server['response_body']['properties']['server']['properties'][
    'image']['oneOf'][0]['properties'].update({'properties': image_properties})

rebuild_server_with_admin_pass = copy.deepcopy(
    servers296.rebuild_server_with_admin_pass)
rebuild_server_with_admin_pass['response_body']['properties']['server'][
    'properties']['image']['oneOf'][0]['properties'].update(
        {'properties': image_properties})

# Below are the unchanged schema in this microversion. We need to keep this
# schema in this file to have the generic way to select the right schema based
# on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.96***
attach_volume = copy.deepcopy(servers296.attach_volume)
show_volume_attachment = copy.deepcopy(servers296.show_volume_attachment)
list_volume_attachments = copy.deepcopy(servers296.list_volume_attachments)
list_servers = copy.deepcopy(servers296.list_servers)
show_server_diagnostics = copy.deepcopy(servers296.show_server_diagnostics)
get_remote_consoles = copy.deepcopy(servers296.get_remote_consoles)
list_tags = copy.deepcopy(servers296.list_tags)
update_all_tags = copy.deepcopy(servers296.update_all_tags)
delete_all_tags = copy.deepcopy(servers296.delete_all_tags)
check_tag_existence = copy.deepcopy(servers296.check_tag_existence)
update_tag = copy.deepcopy(servers296.update_tag)
delete_tag = copy.deepcopy(servers296.delete_tag)
show_instance_action = copy.deepcopy(servers296.show_instance_action)
create_backup = copy.deepcopy(servers296.create_backup)



# Copyright 2015 Fujitsu(fnst) Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

common_snapshot_info = {
    'type': 'object',
    'properties': {
        'id': {'type': 'string'},
        'volumeId': {'type': 'string'},
        'status': {'type': 'string'},
        'size': {'type': 'integer'},
        'createdAt': parameter_types.date_time,
        'displayName': {'type': ['string', 'null']},
        'displayDescription': {'type': ['string', 'null']}
    },
    'additionalProperties': False,
    'required': ['id', 'volumeId', 'status', 'size',
                 'createdAt', 'displayName', 'displayDescription']
}

create_get_snapshot = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'snapshot': common_snapshot_info
        },
        'additionalProperties': False,
        'required': ['snapshot']
    }
}

list_snapshots = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'snapshots': {
                'type': 'array',
                'items': common_snapshot_info
            }
        },
        'additionalProperties': False,
        'required': ['snapshots']
    }
}

delete_snapshot = {
    'status_code': [202]
}



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.volume.v3 import quota_classes_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestQuotaClassesClient(base.BaseServiceTest):

    FAKE_QUOTA_CLASS_SET = {
        "id": "test",
        "gigabytes": 2000,
        "volumes": 200,
        "snapshots": 50,
        "backups": 20,
        "backup_gigabytes": 1500,
        "per_volume_gigabytes": 500,
    }

    def setUp(self):
        super(TestQuotaClassesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = quota_classes_client.QuotaClassesClient(
            fake_auth, 'volume', 'regionOne')

    def _test_show_quota_class_set(self, bytes_body=False):
        fake_body = {'quota_class_set': self.FAKE_QUOTA_CLASS_SET}
        self.check_service_client_function(
            self.client.show_quota_class_set,
            'tempest.lib.common.rest_client.RestClient.get',
            fake_body,
            bytes_body,
            quota_class_id="test")

    def _test_update_quota_class_set(self, bytes_body=False):
        fake_quota_class_set = copy.deepcopy(self.FAKE_QUOTA_CLASS_SET)
        fake_quota_class_set.pop("id")
        fake_body = {'quota_class_set': fake_quota_class_set}
        self.check_service_client_function(
            self.client.update_quota_class_set,
            'tempest.lib.common.rest_client.RestClient.put',
            fake_body,
            bytes_body,
            quota_class_id="test")

    def test_show_quota_class_set_with_str_body(self):
        self._test_show_quota_class_set()

    def test_show_quota_class_set_with_bytes_body(self):
        self._test_show_quota_class_set(bytes_body=True)

    def test_update_quota_class_set_with_str_boy(self):
        self._test_update_quota_class_set()

    def test_update_quota_class_set_with_bytes_body(self):
        self._test_update_quota_class_set(bytes_body=True)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.lib.services.identity.v3 import catalog_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestCatalogClient(base.BaseServiceTest):
    FAKE_CATALOG_INFO = {
        'catalog': [
            {
                'endpoints': [
                    {
                        'id': '39dc322ce86c4111b4f06c2eeae0841b',
                        'interface': 'public',
                        'region': 'RegionOne',
                        'url': 'http://localhost:5000'
                    },
                ],
                'id': 'ac58672276f848a7b1727850b3ebe826',
                'type': 'compute',
                'name': 'nova'
            },
            {
                'endpoints': [
                    {
                        'id': '39dc322ce86c4111b4f06c2eeae0841b',
                        'interface': 'public',
                        'region': 'RegionOne',
                        'url': 'http://localhost:5000'
                    },
                ],
                'id': 'b7c5ed2b486a46dbb4c221499d22991c',
                'type': 'image',
                'name': 'glance'
            },
            {
                'endpoints': [
                    {
                        'id': '39dc322ce86c4111b4f06c2eeae0841b',
                        'interface': 'public',
                        'region': 'RegionOne',
                        'url': 'http://localhost:5000'
                    },
                ],
                'id': '4363ae44bdf34a3981fde3b823cb9aa2',
                'type': 'identity',
                'name': 'keystone'
            }

        ],
        'links': {
            'self': 'http://localhost/identity/v3/catalog',
            'previous': None,
            'next': None
        }
    }

    def setUp(self):
        super(TestCatalogClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = catalog_client.CatalogClient(fake_auth, 'identity',
                                                   'RegionOne')

    def test_show_catalog_with_bytes_body(self):
        self._test_show_catalog(bytes_body=True)

    def test_show_catalog_with_str_body(self):
        self._test_show_catalog()

    def _test_show_catalog(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_catalog,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_CATALOG_INFO,
            bytes_body)



# Copyright 2021 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.scenario import manager

CONF = config.CONF


class ComputeProjectQuotaTest(manager.ScenarioTest):
    """The test base class for compute unified limits tests.

    Dynamic credentials (unique tenants) are created on a per-class basis, so
    we test different quota limits in separate test classes to prevent a quota
    limit update in one test class from affecting a test running in another
    test class in parallel.

    https://docs.openstack.org/tempest/latest/configuration.html#dynamic-credentials
    """
    credentials = ['primary', 'system_admin']
    force_tenant_isolation = True

    @classmethod
    def skip_checks(cls):
        super(ComputeProjectQuotaTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.unified_limits:
            raise cls.skipException('Compute unified limits are not enabled.')

    @classmethod
    def resource_setup(cls):
        super(ComputeProjectQuotaTest, cls).resource_setup()

        # Figure out and record the nova service id
        services = cls.os_system_admin.identity_services_v3_client.\
            list_services()
        nova_services = [x for x in services['services']
                         if x['name'] == 'nova']
        cls.nova_service_id = nova_services[0]['id']

        # Pre-create quota limits in subclasses and record their IDs so we can
        # update them in-place without needing to know which ones have been
        # created and in which order.
        cls.limit_ids = {}

    @classmethod
    def _create_limit(cls, name, value):
        return cls.os_system_admin.identity_limits_client.create_limit(
            CONF.identity.region, cls.nova_service_id,
            cls.servers_client.tenant_id, name, value)['limits'][0]['id']

    def _update_limit(self, name, value):
        self.os_system_admin.identity_limits_client.update_limit(
            self.limit_ids[name], value)


class ServersQuotaTest(ComputeProjectQuotaTest):

    @classmethod
    def resource_setup(cls):
        super(ServersQuotaTest, cls).resource_setup()

        try:
            cls.limit_ids['servers'] = cls._create_limit(
                'servers', 5)
            cls.limit_ids['class:VCPU'] = cls._create_limit(
                'class:VCPU', 10)
            cls.limit_ids['class:MEMORY_MB'] = cls._create_limit(
                'class:MEMORY_MB', 25 * 1024)
            cls.limit_ids['class:DISK_GB'] = cls._create_limit(
                'class:DISK_GB', 10)
        except lib_exc.Forbidden:
            raise cls.skipException('Target system is not configured with '
                                    'compute unified limits')

    @decorators.idempotent_id('555d8bbf-d2ed-4e39-858c-4235899402d9')
    @utils.services('compute')
    def test_server_count_vcpu_memory_disk_quota(self):
        # Set a quota on the number of servers for our tenant to one.
        self._update_limit('servers', 1)

        # Create one server.
        first = self.create_server(name='first')

        # Second server would put us over quota, so expect failure.
        # NOTE: In nova, quota exceeded raises 403 Forbidden.
        self.assertRaises(lib_exc.Forbidden,
                          self.create_server,
                          name='second')

        # Update our limit to two.
        self._update_limit('servers', 2)

        # Now the same create should succeed.
        second = self.create_server(name='second')

        # Third server would put us over quota, so expect failure.
        self.assertRaises(lib_exc.Forbidden,
                          self.create_server,
                          name='third')

        # Delete the first server to put us under quota.
        self.servers_client.delete_server(first['id'])
        waiters.wait_for_server_termination(self.servers_client, first['id'])

        # Now the same create should succeed.
        third = self.create_server(name='third')

        # Set the servers limit back to 10 to test other resources.
        self._update_limit('servers', 10)

        # Default flavor has: VCPU=1, MEMORY_MB=512, DISK_GB=1
        # We are currently using 2 VCPU, set the limit to 2.
        self._update_limit('class:VCPU', 2)

        # Server create should fail as it would go over quota.
        self.assertRaises(lib_exc.Forbidden,
                          self.create_server,
                          name='fourth')

        # Delete the second server to put us under quota.
        self.servers_client.delete_server(second['id'])
        waiters.wait_for_server_termination(self.servers_client, second['id'])

        # Same create should now succeed.
        fourth = self.create_server(name='fourth')

        # We are currently using 2 DISK_GB. Set limit to 1.
        self._update_limit('class:DISK_GB', 1)

        # Server create should fail because we're already over (new) quota.
        self.assertRaises(lib_exc.Forbidden,
                          self.create_server,
                          name='fifth')

        # Delete the third server.
        self.servers_client.delete_server(third['id'])
        waiters.wait_for_server_termination(self.servers_client, third['id'])

        # Server create should fail again because it would still put us over
        # quota.
        self.assertRaises(lib_exc.Forbidden,
                          self.create_server,
                          name='fifth')

        # Delete the fourth server.
        self.servers_client.delete_server(fourth['id'])
        waiters.wait_for_server_termination(self.servers_client, fourth['id'])

        # Server create should succeed now.
        self.create_server(name='fifth')



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.volume.v3 import backups_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestBackupsClient(base.BaseServiceTest):

    FAKE_BACKUP_LIST = {
        "backups": [
            {
                "id": "2ef47aee-8844-490c-804d-2a8efe561c65",
                "links": [
                    {
                        "href": "fake-url-1",
                        "rel": "self"
                    },
                    {
                        "href": "fake-url-2",
                        "rel": "bookmark"
                    }
                ],
                "name": "backup001"
            }
        ]
    }

    FAKE_BACKUP_LIST_WITH_DETAIL = {
        "backups": [
            {
                "availability_zone": "az1",
                "container": "volumebackups",
                "created_at": "2013-04-02T10:35:27.000000",
                "updated_at": "2013-04-02T10:39:27.000000",
                "data_timestamp": "2013-04-02T10:35:27.000000",
                "description": None,
                "fail_reason": None,
                "id": "2ef47aee-8844-490c-804d-2a8efe561c65",
                "links": [
                    {
                        "href": "fake-url-1",
                        "rel": "self"
                    },
                    {
                        "href": "fake-url-2",
                        "rel": "bookmark"
                    }
                ],
                "name": "backup001",
                "object_count": 22,
                "os-backup-project-attr:project_id": "2c67a14be9314c5dae2ee6",
                "user_id": "515ba0dd59f84f25a6a084a45d8d93b2",
                "size": 1,
                "status": "available",
                "volume_id": "e5185058-943a-4cb4-96d9-72c184c337d6",
                "is_incremental": True,
                "has_dependent_backups": False
            }
        ]
    }

    FAKE_BACKUP_UPDATE = {
        "backup": {
            "id": "4c65c15f-a5c5-464b-b92a-90e4c04636a7",
            "name": "fake-backup-name",
            "links": [
                {
                    "href": "fake-url-1",
                    "rel": "self"
                },
                {
                    "href": "fake-url-2",
                    "rel": "bookmark"
                }
            ]
        }
    }

    def setUp(self):
        super(TestBackupsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = backups_client.BackupsClient(fake_auth,
                                                   'volume',
                                                   'regionOne')

    def _test_list_backups(self, detail=False, mock_args='backups',
                           bytes_body=False, **params):
        if detail:
            resp_body = self.FAKE_BACKUP_LIST_WITH_DETAIL
        else:
            resp_body = self.FAKE_BACKUP_LIST
        self.check_service_client_function(
            self.client.list_backups,
            'tempest.lib.common.rest_client.RestClient.get',
            resp_body,
            to_utf=bytes_body,
            mock_args=[mock_args],
            detail=detail,
            **params)

    def test_list_backups_with_str_body(self):
        self._test_list_backups()

    def test_list_backups_with_bytes_body(self):
        self._test_list_backups(bytes_body=True)

    def test_list_backups_with_detail_with_str_body(self):
        mock_args = "backups/detail"
        self._test_list_backups(detail=True, mock_args=mock_args)

    def test_list_backups_with_detail_with_bytes_body(self):
        mock_args = "backups/detail"
        self._test_list_backups(detail=True, mock_args=mock_args,
                                bytes_body=True)

    def test_list_backups_with_params(self):
        # Run the test separately for each param, to avoid assertion error
        # resulting from randomized params order.
        mock_args = 'backups?sort_key=name'
        self._test_list_backups(mock_args=mock_args, sort_key='name')

        mock_args = 'backups/detail?limit=10'
        self._test_list_backups(detail=True, mock_args=mock_args,
                                bytes_body=True, limit=10)

    def _test_update_backup(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_backup,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_BACKUP_UPDATE,
            bytes_body,
            backup_id='4c65c15f-a5c5-464b-b92a-90e4c04636a7')

    def test_update_backup_with_str_body(self):
        self._test_update_backup()

    def test_update_backup_with_bytes_body(self):
        self._test_update_backup(bytes_body=True)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class VolumesTestJSON(base.BaseV2ComputeTest):
    """Test listing volumes with compute microversion less than 2.36"""

    # NOTE: This test creates a number of 1G volumes. To run successfully,
    # ensure that the backing file for the volume group that Nova uses
    # has space for at least 3 1G volumes!
    # If you are running a Devstack environment, ensure that the
    # VOLUME_BACKING_FILE_SIZE is at least 4G in your localrc

    # These tests will fail with a 404 starting from microversion 2.36. For
    # more information, see:
    # https://docs.openstack.org/api-ref/compute/#volume-extension-os-volumes-os-snapshots-deprecated
    max_microversion = '2.35'

    @classmethod
    def skip_checks(cls):
        super(VolumesTestJSON, cls).skip_checks()
        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(VolumesTestJSON, cls).setup_clients()
        cls.client = cls.volumes_extensions_client

    @classmethod
    def resource_setup(cls):
        super(VolumesTestJSON, cls).resource_setup()
        # Create 3 Volumes
        cls.volume_list = []
        for _ in range(3):
            metadata = {'Type': 'work'}
            volume = cls.create_volume(metadata=metadata)
            volume = cls.client.show_volume(volume['id'])['volume']
            cls.volume_list.append(volume)

    @decorators.idempotent_id('bc2dd1a0-15af-48e5-9990-f2e75a48325d')
    def test_volume_list(self):
        """Test listing volumes should return all volumes"""
        # Fetch all Volumes
        fetched_list = self.client.list_volumes()['volumes']
        # Now check if all the Volumes created in setup are in fetched list
        missing_volumes = [
            v for v in self.volume_list if v not in fetched_list
        ]

        self.assertFalse(missing_volumes,
                         "Failed to find volume %s in fetched list" %
                         ', '.join(m_vol['displayName']
                                   for m_vol in missing_volumes))

    @decorators.idempotent_id('bad0567a-5a4f-420b-851e-780b55bb867c')
    def test_volume_list_with_details(self):
        """Test listing volumes with detail should return all volumes"""
        # Fetch all Volumes
        fetched_list = self.client.list_volumes(detail=True)['volumes']
        # Now check if all the Volumes created in setup are in fetched list
        missing_volumes = [
            v for v in self.volume_list if v not in fetched_list
        ]

        self.assertFalse(missing_volumes,
                         "Failed to find volume %s in fetched list" %
                         ', '.join(m_vol['displayName']
                                   for m_vol in missing_volumes))

    @decorators.idempotent_id('1048ed81-2baf-487a-b284-c0622b86e7b8')
    def test_volume_list_param_limit(self):
        """Test listing volumes based on limit set

        If we list volumes with limit=2, then only 2 volumes should be
        returned.
        """
        params = {'limit': 2}
        fetched_vol_list = self.client.list_volumes(**params)['volumes']

        self.assertEqual(len(fetched_vol_list), params['limit'],
                         "Failed to list volumes by limit set")

    @decorators.idempotent_id('33985568-4965-49d5-9bcc-0aa007ca5b7a')
    def test_volume_list_with_detail_param_limit(self):
        """Test listing volumes with detail based on limit set

        If we list volumes with detail with limit=2, then only 2 volumes with
        detail should be returned.
        """
        params = {'limit': 2}
        fetched_vol_list = self.client.list_volumes(detail=True,
                                                    **params)['volumes']

        self.assertEqual(len(fetched_vol_list), params['limit'],
                         "Failed to list volume details by limit set")

    @decorators.idempotent_id('51c22651-a074-4ea7-af0b-094f9331303e')
    def test_volume_list_param_offset_and_limit(self):
        """Test listing volumes based on offset and limit set

        If we list volumes with offset=1 and limit=1, then 1 volume located
        in the position 1 in the all volumes list should be returned.
        (The items in the all volumes list start from position 0.)
        """
        # get all volumes list
        all_vol_list = self.client.list_volumes()['volumes']
        params = {'offset': 1, 'limit': 1}
        fetched_vol_list = self.client.list_volumes(**params)['volumes']

        # Validating length of the fetched volumes
        self.assertEqual(len(fetched_vol_list), params['limit'],
                         "Failed to list volumes by offset and limit")
        # Validating offset of fetched volume
        for index, volume in enumerate(fetched_vol_list):
            self.assertEqual(volume['id'],
                             all_vol_list[index + params['offset']]['id'],
                             "Failed to list volumes by offset and limit")

    @decorators.idempotent_id('06b6abc4-3f10-48e9-a7a1-3facc98f03e5')
    def test_volume_list_with_detail_param_offset_and_limit(self):
        """Test listing volumes with detail based on offset and limit set

        If we list volumes with detail with offset=1 and limit=1, then 1
        volume with detail located in the position 1 in the all volumes list
        should be returned.
        (The items in the all volumes list start from position 0.)
        """
        # get all volumes list
        all_vol_list = self.client.list_volumes(detail=True)['volumes']
        params = {'offset': 1, 'limit': 1}
        fetched_vol_list = self.client.list_volumes(detail=True,
                                                    **params)['volumes']

        # Validating length of the fetched volumes
        self.assertEqual(len(fetched_vol_list), params['limit'],
                         "Failed to list volume details by offset and limit")
        # Validating offset of fetched volume
        for index, volume in enumerate(fetched_vol_list):
            self.assertEqual(volume['id'],
                             all_vol_list[index + params['offset']]['id'],
                             "Failed to list volume details by "
                             "offset and limit")
